# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_quantizationtools_MeasurewiseQSchemaItem___new___01():

    item = quantizationtools.MeasurewiseQSchemaItem()

    assert item.beatspan is None
    assert item.search_tree is None
    assert item.tempo is None
    assert item.time_signature is None


def test_quantizationtools_MeasurewiseQSchemaItem___new___02():

    item = quantizationtools.MeasurewiseQSchemaItem(
        tempo=((1, 4), 60)
        )

    assert item.beatspan is None
    assert item.search_tree is None
    assert item.tempo == Tempo((1, 4), 60)
    assert item.time_signature is None


def test_quantizationtools_MeasurewiseQSchemaItem___new___03():

    item = quantizationtools.MeasurewiseQSchemaItem(
        time_signature=(6, 8)
        )

    assert item.beatspan == durationtools.Duration(1, 8)
    assert item.search_tree is None
    assert item.tempo is None
    assert item.time_signature == TimeSignature((6, 8))


def test_quantizationtools_MeasurewiseQSchemaItem___new___04():

    item = quantizationtools.MeasurewiseQSchemaItem(
        tempo=((1, 4), 57),
        time_signature=(6, 8),
        )

    assert item.beatspan == durationtools.Duration(1, 8)
    assert item.search_tree is None
    assert item.tempo == Tempo((1, 4), 57)
    assert item.time_signature == TimeSignature((6, 8))


def test_quantizationtools_MeasurewiseQSchemaItem___new___05():

    tempo = Tempo(textual_indication='lento')
    pytest.raises(
        AssertionError,
        'item = quantizationtools.MeasurewiseQSchemaItem(tempo=tempo)',
        )



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class StringContactPoint(AbjadValueObject):
    r'''String contact point.

    ..  container:: example

        **Example 1.** Sul ponticello:

        ::

            >>> indicator = indicatortools.StringContactPoint('sul ponticello')
            >>> print(format(indicator))
            indicatortools.StringContactPoint(
                contact_point='sul ponticello',
                )

    ..  container:: example

        **Example 2.** Sul tasto:

        ::

            >>> indicator = indicatortools.StringContactPoint('sul tasto')
            >>> print(format(indicator))
            indicatortools.StringContactPoint(
                contact_point='sul tasto',
                )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contact_point',
        '_default_scope',
        )

    _contact_points = (
        'dietro ponticello',
        'molto sul ponticello',
        'molto sul tasto',
        'ordinario',
        'pizzicato',
        'ponticello',
        'sul ponticello',
        'sul tasto',
        )

    _contact_point_abbreviations = {
        'dietro ponticello': 'd.p.',
        'molto sul ponticello': 'm.s.p',
        'molto sul tasto': 'm.s.t.',
        'ordinario': 'ord.',
        'pizzicato': 'pizz.',
        'ponticello': 'p.',
        'sul ponticello': 's.p.',
        'sul tasto': 's.t.',
        }

    ### INITIALIZER ###

    def __init__(
        self,
        contact_point='ordinario',
        ):
        contact_point = str(contact_point)
        assert contact_point in self._contact_points
        self._contact_point = contact_point
        self._default_scope = None

    ### PUBLIC PROPERTIES ###

    @property
    def contact_point(self):
        r'''Gets contact point of string contact point.

        ..  container:: example

            **Example 1.** Sul ponticello:

            ::

                >>> indicator = indicatortools.StringContactPoint('sul ponticello')
                >>> indicator.contact_point
                'sul ponticello'

        ..  container:: example

            **Example 2.** Sul tasto:

            ::

                >>> indicator = indicatortools.StringContactPoint('sul tasto')
                >>> indicator.contact_point
                'sul tasto'

        Set to known string.

        Defaults to ``'ordinario'``.

        Returns known string.
        '''
        return self._contact_point


    @property
    def default_scope(self):
        r'''Gets default scope of string contact point.

        ..  container:: example

            **Example 1.** Sul ponticello:

            ::

                >>> indicator = indicatortools.StringContactPoint('sul ponticello')
                >>> indicator.default_scope is None
                True

        ..  container:: example

            **Example 2.** Sul tasto:

            ::

                >>> indicator = indicatortools.StringContactPoint('sul tasto')
                >>> indicator.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def markup(self):
        r'''Gets markup of string contact point.

        ..  container:: example

            **Example 1.** Sul ponticello:

            ::

                >>> indicator = indicatortools.StringContactPoint('sul ponticello')
                >>> show(indicator.markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(indicator.markup))
                \markup {
                    \caps
                        S.P.
                    }

        ..  container:: example

            **Example 2.** Sul tasto:

            ::

                >>> indicator = indicatortools.StringContactPoint('sul tasto')
                >>> show(indicator.markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(indicator.markup))
                \markup {
                    \caps
                        S.T.
                    }

        Returns abbreviation markup.
        '''
        from abjad.tools import markuptools
        markup = self._contact_point_abbreviations[self.contact_point]
        markup = markup.title()
        markup = markuptools.Markup(markup)
        markup = markup.caps()
        return markup



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__contexts__with_blocks_01():

    target = Staff([])

    r'''
    \new Staff {
    }
    '''

    string = r'''\new Staff \with { } {
    }
    '''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_get_indicators_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, staff[:])
    command_1 = indicatortools.LilyPondCommand('slurDotted')
    attach(command_1, staff[0])
    command_2 = indicatortools.LilyPondCommand('slurUp')
    attach(command_2, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \slurDotted
            \slurUp
            c'8 (
            d'8
            e'8
            f'8 )
        }
        '''
        ), format(staff)

    indicators = inspect_(staff[0]).get_indicators(
        indicatortools.LilyPondCommand)
    assert command_1 in indicators
    assert command_2 in indicators
    assert len(indicators) == 2


def test_agenttools_InspectionAgent_get_indicators_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, staff[:])
    comment = indicatortools.LilyPondComment('beginning of note content')
    attach(comment, staff[0])
    command = indicatortools.LilyPondCommand('slurDotted')
    attach(command, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            % beginning of note content
            \slurDotted
            c'8 (
            d'8
            e'8
            f'8 )
        }
        '''
        ), format(staff)

    items = inspect_(staff[0]).get_indicators()
    assert comment in items
    assert command in items
    assert len(items) == 2


def test_agenttools_InspectionAgent_get_indicators_03():

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef('treble')
    attach(clef, staff)
    dynamic = Dynamic('p')
    attach(dynamic, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \clef "treble"
            c'8 \p
            d'8
            e'8
            f'8
        }
        '''
        ), format(staff)

    indicators = inspect_(staff[0]).get_indicators()
    assert dynamic == indicators[0]
    assert len(indicators) == 1


def test_agenttools_InspectionAgent_get_indicators_04():

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef('treble')
    attach(clef, staff)
    dynamic = Dynamic('p')
    attach(dynamic, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \clef "treble"
            c'8 \p
            d'8
            e'8
            f'8
        }
        '''
        ), format(staff)

    dynamics = inspect_(staff[0]).get_indicators(Dynamic)
    assert dynamic == dynamics[0]
    assert len(dynamics) == 1


def test_agenttools_InspectionAgent_get_indicators_05():

    staff = Staff("c'8 d'8 e'8 f'8")
    annotation_1 = indicatortools.Annotation('annotation 1')
    attach(annotation_1, staff[0])
    annotation_2 = indicatortools.Annotation('annotation 2')
    attach(annotation_2, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        ), format(staff)

    annotations = inspect_(staff[0]).get_indicators(indicatortools.Annotation)
    assert annotations == (annotation_1, annotation_2)


def test_agenttools_InspectionAgent_get_indicators_06():

    staff = Staff("c'8 d'8 e'8 f'8")
    comment_1 = indicatortools.LilyPondComment('comment 1')
    attach(comment_1, staff[0])
    comment_2 = indicatortools.LilyPondComment('comment 2')
    attach(comment_2, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            % comment 1
            % comment 2
            c'8
            d'8
            e'8
            f'8
        }
        '''
        ), format(staff)

    indicators = inspect_(staff[0]).get_indicators(
        indicatortools.LilyPondComment)
    assert comment_1 in indicators
    assert comment_2 in indicators
    assert len(indicators) == 2


def test_agenttools_InspectionAgent_get_indicators_07():

    note = Note("c'4")
    stem_tremolo = indicatortools.StemTremolo(16)
    attach(stem_tremolo, note)
    stem_tremolos = inspect_(note).get_indicators(indicatortools.StemTremolo)

    assert stem_tremolos[0] is stem_tremolo


# -*- coding: utf-8 -*-


def timespan_2_stops_before_timespan_1_stops(
    timespan_1=None,
    timespan_2=None,
    hold=False,
    ):
    r'''Makes time relation indicating that `timespan_2`
    happens during `timespan_1`.

    ::

        >>> time_relation = timespantools.timespan_2_stops_before_timespan_1_stops()
        >>> print(format(time_relation))
        timespantools.TimespanTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.Inequality('timespan_2.stop_offset < timespan_1.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    Returns time relation or boolean.
    '''
    from abjad.tools import timespantools

    time_relation = timespantools.TimespanTimespanTimeRelation(
        timespantools.CompoundInequality([
            'timespan_2.stop_offset < timespan_1.stop_offset',
            ]),
        timespan_1=timespan_1,
        timespan_2=timespan_2)

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



# -*- coding: utf-8 -*-
import re
import six
from abjad.tools.stringtools.is_snake_case \
    import underscore_delimited_lowercase_regex_body


underscore_delimited_lowercase_package_regex_body = """
    ({}\.)*
    {}
    """.format(
        underscore_delimited_lowercase_regex_body,
        underscore_delimited_lowercase_regex_body
        )

underscore_delimited_lowercase_package_regex = re.compile(
    '^{}$'.format(underscore_delimited_lowercase_package_regex_body),
    re.VERBOSE,
    )


def is_snake_case_package_name(argument):
    r'''Is true when `argument` is a string and is underscore-delimited lowercase
    package name.

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_package_name('foo.bar.blah_package')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_package_name('foo.bar.BlahPackage')
            False

    Returns true or false.
    '''
    if not isinstance(argument, six.string_types):
        return False
    return bool(underscore_delimited_lowercase_package_regex.match(argument))



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitchClass___sub___01():

    mdi = pitchtools.NamedPitchClass('c') - pitchtools.NamedPitchClass('d')
    assert mdi == pitchtools.NamedInversionEquivalentIntervalClass('major', 2)

    mdi = pitchtools.NamedPitchClass('d') - pitchtools.NamedPitchClass('c')
    assert mdi == pitchtools.NamedInversionEquivalentIntervalClass('major', 2)


def test_pitchtools_NamedPitchClass___sub___02():

    mdi = pitchtools.NamedPitchClass('c') - pitchtools.NamedPitchClass('cf')
    assert mdi == pitchtools.NamedInversionEquivalentIntervalClass('augmented', 1)

    mdi = pitchtools.NamedPitchClass('cf') - pitchtools.NamedPitchClass('c')
    assert mdi == pitchtools.NamedInversionEquivalentIntervalClass('augmented', 1)



# -*- coding: utf-8 -*-
import numbers
from abjad.tools import mathtools
from abjad.tools.pitchtools.IntervalClass import IntervalClass


class NumberedIntervalClass(IntervalClass):
    '''Numbered interval-class.

    ..  container:: example

        Initializes from integer:

        ::

            >>> NumberedIntervalClass(-14)
            NumberedIntervalClass(-2)

    ..  container:: example

        Initializes from float:

        ::

            >>> NumberedIntervalClass(-14.5)
            NumberedIntervalClass(-2.5)

    ..  container:: example

        Initializes from string:

        ::

            >>> NumberedIntervalClass('-14.5')
            NumberedIntervalClass(-2.5)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, item=None):
        from abjad.tools import pitchtools
        if isinstance(item, numbers.Number):
            sign = mathtools.sign(item)
            abs_token = abs(item)
            if abs_token % 12 == 0 and 12 <= abs_token:
                number = 12
            else:
                number = abs_token % 12
            number *= sign
        elif isinstance(item, pitchtools.Interval):
            number = item.semitones
            sign = mathtools.sign(number)
            abs_number = abs(number)
            if abs_number % 12 == 0 and 12 <= abs_number:
                number = 12
            else:
                number = abs_number % 12
            number *= sign
        elif isinstance(item, pitchtools.IntervalClass):
            number = item.number
            sign = mathtools.sign(number)
            abs_number = abs(number)
            if abs_number % 12 == 0 and 12 <= abs_number:
                number = 12
            else:
                number = abs_number % 12
            number *= sign
        elif item is None:
            number = 0
        elif isinstance(item, str):
            number = float(item)
            if mathtools.is_integer_equivalent(number):
                number = int(number)
            sign = mathtools.sign(number)
            abs_token = abs(number)
            if abs_token % 12 == 0 and 12 <= abs_token:
                number = 12
            else:
                number = abs_token % 12
            number *= sign
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, item)
            raise ValueError(message)
        self._number = number

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of numbered interval-class.

        Returns new numbered interval-class.
        '''
        return type(self)(abs(self._number))

    def __eq__(self, argument):
        r'''Is true when `argument` is a numbered interval-class with number equal to
        that of this numbered interval-class. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.number == argument.number:
                return True
        return False

    def __float__(self):
        r'''Changes numbered interval-class to float.

        Returns float.
        '''
        return float(self._number)

    def __hash__(self):
        r'''Hashes numbered interval-class.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NumberedIntervalClass, self).__hash__()

    def __int__(self):
        r'''Changes numbered interval-class to integer.

        Returns integer.
        '''
        return self._number

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        return '{}{}'.format(self.direction_symbol, abs(self.number))

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        r'''Direction number of numbered interval-class.

        Returns -1, 0 or 1.
        '''
        if self.number < 1:
            return -1
        elif self.number == 1:
            return 0
        else:
            return 1

    @property
    def direction_symbol(self):
        r'''Direction symbol of numbered interval class.

        Returns string.
        '''
        if self.number < 1:
            return '-'
        else:
            return '+'

    @property
    def direction_word(self):
        r'''Direction word of numbered interval-class.

        Returns string.
        '''
        if self.number < 1:
            return 'descending'
        elif self.number == 1:
            return ''
        else:
            return 'ascending'

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        '''Makes numbered interval-class from `pitch_carrier_1` and
        `pitch_carrier_2`.

        ::

            >>> NumberedIntervalClass.from_pitch_carriers(
            ...     NamedPitch(-2),
            ...     NamedPitch(12),
            ...     )
            NumberedIntervalClass(2)

        Returns numbered interval-class.
        '''
        from abjad.tools import pitchtools
        # get numbered interval
        interval = pitchtools.NumberedInterval.from_pitch_carriers(
            pitch_carrier_1, pitch_carrier_2)
        # return numbered interval-class
        return class_(interval)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_RegistrationComponent_target_octave_start_pitch_01():

    component = pitchtools.RegistrationComponent('[A0, C8]', 15)
    assert component.target_octave_start_pitch == pitchtools.NumberedPitch(15)



# -*- coding: utf-8 -*-
from abjad.tools.spannertools.Hairpin import Hairpin


class Decrescendo(Hairpin):
    r'''Decrescendo.

    ..  container:: example

        ::

            >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
            >>> decrescendo = spannertools.Decrescendo()
            >>> attach(decrescendo, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                r4
                c'8 \>
                d'8
                e'8
                f'8 \!
                r4
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        include_rests=False,
        overrides=None,
        ):
        Hairpin.__init__(
            self,
            descriptor='>',
            direction=direction,
            include_rests=include_rests,
            overrides=overrides,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def descriptor(self):
        r'''Gets descriptor of decrescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> decrescendo = Decrescendo()
                >>> attach(decrescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> decrescendo.descriptor
                '>'

        Returns string.
        '''
        return self._descriptor

    @property
    def direction(self):
        r'''Gets direction.

        ..  container:: example

            Positions decrescendo above staff:

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> decrescendo = Decrescendo(direction=Up)
                >>> attach(decrescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                r4
                c'8 ^ \>
                d'8
                e'8
                f'8 \!
                r4
            }

            ::

                >>> decrescendo.direction
                '^'

        Defaults to none.

        Set to up, down or none.

        Returns up, down or none.
        '''
        return self._direction

    @property
    def include_rests(self):
        r'''Gets include-rests flag of decrescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> decrescendo = Decrescendo(include_rests=True)
                >>> attach(decrescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                r4 \>
                c'8
                d'8
                e'8
                f'8
                r4 \!
            }

            ::

                >>> decrescendo.include_rests
                True

        Returns true or false.
        '''
        return self._include_rests

    @property
    def shape_string(self):
        r'''Gets shape string of decrescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> decrescendo = Decrescendo()
                >>> attach(decrescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> decrescendo.shape_string
                '>'

        Returns string.
        '''
        return self._shape_string

    @property
    def start_dynamic(self):
        r'''Gets start dynamic of decrescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> decrescendo = Decrescendo()
                >>> attach(decrescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> decrescendo.start_dynamic is None
                True

        Returns dynamic or none.
        '''
        return self._start_dynamic

    @property
    def stop_dynamic(self):
        r'''Gets stop dynamic of decrescendo.

        ..  container:: example

            ::

                >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
                >>> decrescendo = Decrescendo()
                >>> attach(decrescendo, staff[:])
                >>> show(staff) # doctest: +SKIP

            ::

                >>> decrescendo.stop_dynamic is None
                True

        Returns dynamic or none.
        '''
        return self._stop_dynamic



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Inversion(AbjadValueObject):
    r'''Inversion operator.

    ..  container:: example

        ::

            >>> Inversion()
            Inversion()

    ..  container:: example

        ::

            >>> Inversion(axis=15)
            Inversion(axis=NamedPitch("ef''"))

    Object model of twelve-tone inversion operator.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        '_axis',
        )

    ### INITIALIZER ###

    def __init__(self, axis=None):
        from abjad.tools import pitchtools
        if axis is not None:
            axis = pitchtools.NamedPitch(axis)
        self._axis = axis

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r'''Composes inversion and `operator`.

        ..  container:: example

            Example segment:

            ::

                >>> items = [0, 2, 4, 5]
                >>> segment = PitchClassSegment(items=items)
                >>> show(segment) # doctest: +SKIP

            Example operators:

            ::

                >>> inversion = Inversion()
                >>> transposition = Transposition(n=3)

        ..  container:: example

            Transposition followed by inversion:

            ::

                >>> operator = inversion + transposition
                >>> str(operator)
                'IT3'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    a'8
                    g'8
                    f'8
                    e'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Inversion followed by transposition:

            ::

                >>> operator = transposition + inversion
                >>> str(operator)
                'T3I'

            ::

                >>> segment_ = operator(segment)
                >>> show(segment_) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = segment_.__illustrate__()
                >>> f(lilypond_file[Voice])
                \new Voice {
                    ef'8
                    cs'8
                    b'8
                    bf'8
                    \bar "|."
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Returns compound operator:

            ::

                >>> f(operator)
                pitchtools.CompoundOperator(
                    operators=[
                        pitchtools.Inversion(),
                        pitchtools.Transposition(
                            n=3,
                            ),
                        ],
                    )

        '''
        from abjad.tools import pitchtools
        return pitchtools.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        r'''Calls inversion on `argument`.

        ..  container:: example

            Inverts numbered pitch-class:

            ::

                >>> inversion = Inversion()
                >>> pitch_class = NumberedPitchClass(1)
                >>> inversion(pitch_class)
                NumberedPitchClass(11)

        ..  container:: example

            Inverts numbered pitch:

            ::

                >>> inversion = Inversion()
                >>> pitch = NumberedPitch(15)
                >>> inversion(pitch)
                NumberedPitch(-15)

        ..  container:: example

            Inverts named pitch:

            ::

                >>> inversion = Inversion()
                >>> pitch = NamedPitch("d'")
                >>> inversion(pitch)
                NamedPitch('bf')

        ..  container:: example

            Inverts named pitch class:

            ::

                >>> inversion = Inversion()
                >>> pitch_class = NamedPitchClass('d')
                >>> inversion(pitch_class)
                NamedPitchClass('bf')

        ..  container:: example

            Inverts pitch segment:

            ::

                >>> inversion = Inversion()
                >>> segment = PitchSegment("c' d' e'")
                >>> inversion(segment)
                PitchSegment("c' bf af")

        ..  container:: example

            Inverts pitch class segment:

            ::

                >>> inversion = Inversion()
                >>> segment = PitchClassSegment("c d e")
                >>> inversion(segment)
                PitchClassSegment("c bf af")

        ..  container:: example
        
            Inverts pitch class set:

            ::

                >>> inversion = Inversion()
                >>> set_ = PitchClassSet("c d e")
                >>> inversion(set_)
                PitchClassSet(['c', 'af', 'bf'])

        Returns new object with type equal to that of `argument`.
        '''
        if hasattr(argument, 'invert'):
            result = argument.invert(axis=self.axis)
        else:
            message = 'do not know how to invert: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return result

    def __str__(self):
        r'''Gets string representation of operator.

        ..  container:: example

            ::

                >>> str(Inversion())
                'I'

        ..  container:: example

            ::

                >>> str(Inversion(axis=15))
                'I(Eb5)'

        '''
        if self.axis is None:
            return 'I'
        string = 'I({})'
        string = string.format(self.axis.pitch_class_octave_label)
        return string

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        from abjad.tools import markuptools
        markup = markuptools.Markup('I', direction=direction)
        if self.axis is not None:
            axis = self.axis.pitch_class_octave_label
            subscript = markuptools.Markup(axis).sub()
            markup = markuptools.Markup.concat([markup, subscript])
        return markup

    def _is_identity_operator(self):
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def axis(self):
        r'''Gets axis of inversion.

        ..  container:: example

            ::

                >>> inversion = Inversion()
                >>> inversion.axis is None
                True

        ..  container:: example

            ::

                >>> inversion = Inversion(axis=15)
                >>> inversion.axis
                NamedPitch("ef''")

        Returns named pitch or none.
        '''
        return self._axis



# -*- coding: utf-8 -*-
from abjad.tools import graphtools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import spannertools
from abjad.tools.datastructuretools.TreeContainer import TreeContainer
from abjad.tools.rhythmtreetools.RhythmTreeMixin import RhythmTreeMixin


class RhythmTreeContainer(RhythmTreeMixin, TreeContainer):
    r'''A rhythm-tree container.

    ..  container:: example

        **Example 1.** Initializes a rhythm-tree container:

        ::

            >>> container = rhythmtreetools.RhythmTreeContainer(
            ...     preprolated_duration=1, children=[])
            >>> print(format(container))
            rhythmtreetools.RhythmTreeContainer(
                preprolated_duration=durationtools.Duration(1, 1),
                )

    ..  container:: example

        **Example 2.** Similar to Abjad containers, `RhythmTreeContainer`
        supports a list interface, and can be appended, extended, indexed and
        so forth by other `RhythmTreeMixin` subclasses:

        ::

            >>> leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
            >>> leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
            >>> container.extend([leaf_a, leaf_b])
            >>> print(format(container))
            rhythmtreetools.RhythmTreeContainer(
                children=(
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(1, 1),
                        is_pitched=True,
                        ),
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(2, 1),
                        is_pitched=True,
                        ),
                    ),
                preprolated_duration=durationtools.Duration(1, 1),
                )

        ::

            >>> another_container = rhythmtreetools.RhythmTreeContainer(
            ...     preprolated_duration=2)
            >>> another_container.append(
            ...     rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3))
            >>> another_container.append(container[1])
            >>> container.append(another_container)
            >>> print(format(container))
            rhythmtreetools.RhythmTreeContainer(
                children=(
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(1, 1),
                        is_pitched=True,
                        ),
                    rhythmtreetools.RhythmTreeContainer(
                        children=(
                            rhythmtreetools.RhythmTreeLeaf(
                                preprolated_duration=durationtools.Duration(3, 1),
                                is_pitched=True,
                                ),
                            rhythmtreetools.RhythmTreeLeaf(
                                preprolated_duration=durationtools.Duration(2, 1),
                                is_pitched=True,
                                ),
                            ),
                        preprolated_duration=durationtools.Duration(2, 1),
                        ),
                    ),
                preprolated_duration=durationtools.Duration(1, 1),
                )

    ..  container:: example

        **Example 3.** Call `RhythmTreeContainer` with a preprolated_duration
        to generate a tuplet structure:

        ::

            >>> list_ = container((1, 4))
            >>> list_
            [FixedDurationTuplet(Duration(1, 4), "c'8 { 4/5 c'8. c'8 }")]
            >>> tuplet = list_[0]
            >>> show(tuplet) # doctest: +SKIP

        ..  doctest::

            >>> print(format(_[0]))
            \times 2/3 {
                c'8
                \times 4/5 {
                    c'8.
                    c'8
                }
            }

    Returns `RhythmTreeContainer` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_duration',
        '_offset',
        '_offsets_are_current',
        )

    ### INITIALIZER ###

    def __init__(self, children=None, preprolated_duration=1, name=None):
        TreeContainer.__init__(self, name=name)
        RhythmTreeMixin.__init__(self, preprolated_duration=preprolated_duration)
        if isinstance(children, (list, str, tuple)):
            self.extend(children)
        elif children is not None:
            message = 'can not instantiate {} with {!r}.'
            raise ValueError(message.format(type(self), children))

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Concatenate containers self and argument. The operation c = a + b
        returns a new RhythmTreeContainer c with the content of both a and b,
        and a preprolated_duration equal to the sum of the durations
        of a and b. The operation is non-commutative: the content of the
        first operand will be placed before the content of the second operand:

        ::

            >>> a = rhythmtreetools.RhythmTreeParser()('(1 (1 1 1))')[0]
            >>> b = rhythmtreetools.RhythmTreeParser()('(2 (3 4))')[0]

        ::

            >>> c = a + b

        ::

            >>> c.preprolated_duration
            Duration(3, 1)

        ::

            >>> print(format(c))
            rhythmtreetools.RhythmTreeContainer(
                children=(
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(1, 1),
                        is_pitched=True,
                        ),
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(1, 1),
                        is_pitched=True,
                        ),
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(1, 1),
                        is_pitched=True,
                        ),
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(3, 1),
                        is_pitched=True,
                        ),
                    rhythmtreetools.RhythmTreeLeaf(
                        preprolated_duration=durationtools.Duration(4, 1),
                        is_pitched=True,
                        ),
                    ),
                preprolated_duration=durationtools.Duration(3, 1),
                )

        Returns new RhythmTreeContainer.
        '''
        from abjad.tools.rhythmtreetools.RhythmTreeParser \
            import RhythmTreeParser
        if isinstance(argument, str):
            argument = RhythmTreeParser()(argument)
            assert 1 == len(argument) and isinstance(argument[0], type(self))
            argument = argument[0]
        container = type(self)(
            preprolated_duration=self.preprolated_duration +
            argument.preprolated_duration)
        container.extend(self[:])
        container.extend(argument[:])
        return container

    def __call__(self, pulse_duration):
        r'''Generate Abjad score components:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree((1, 4))
            [FixedDurationTuplet(Duration(1, 4), "c'16 { 2/3 c'16 c'16 c'16 } c'8")]

        Returns sequence of components.
        '''
        def recurse(node, tuplet_duration):
            basic_prolated_duration = tuplet_duration / node._contents_duration
            basic_written_duration = \
                basic_prolated_duration.equal_or_greater_power_of_two
            tuplet = scoretools.FixedDurationTuplet(tuplet_duration, [])
            for child in node.children:
                if isinstance(child, type(self)):
                    tuplet.extend(recurse(
                        child,
                        child.preprolated_duration * basic_written_duration))
                else:
                    leaves = child(basic_written_duration)
                    tuplet.extend(leaves)
                    if 1 < len(leaves):
                        tie = spannertools.Tie()
                        attach(tie, leaves)
            if tuplet.multiplier == 1:
                return tuplet[:]
            return [tuplet]
        from abjad.tools.topleveltools import attach
        pulse_duration = durationtools.Duration(pulse_duration)
        assert 0 < pulse_duration
        result = recurse(self, pulse_duration * self.preprolated_duration)
        for component in result[:]:
            if isinstance(component, scoretools.Tuplet):
                if component.is_trivial:
                    component._extract()
        return result

    def __graph__(self, **keywords):
        r'''The GraphvizGraph representation of the RhythmTreeContainer:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> graph = tree.__graph__()
            >>> print(str(graph))
            digraph G {
                graph [bgcolor=transparent,
                    truecolor=true];
                node_0 [label="1",
                    shape=triangle];
                node_1 [label="1",
                    shape=box];
                node_2 [label="2",
                    shape=triangle];
                node_3 [label="1",
                    shape=box];
                node_4 [label="1",
                    shape=box];
                node_5 [label="1",
                    shape=box];
                node_6 [label="2",
                    shape=box];
                node_0 -> node_1;
                node_0 -> node_2;
                node_0 -> node_6;
                node_2 -> node_3;
                node_2 -> node_4;
                node_2 -> node_5;
            }

        ::

            >>> topleveltools.graph(graph) # doctest: +SKIP

        Return `GraphvizGraph` instance.
        '''
        graph = graphtools.GraphvizGraph(
            name='G',
            attributes={
                'bgcolor': 'transparent',
                'truecolor': True,
                },
            )
        node_mapping = {}
        for node in self.nodes:
            graphviz_node = graphtools.GraphvizNode()
            graphviz_node.attributes['label'] = str(node.preprolated_duration)
            if isinstance(node, type(self)):
                graphviz_node.attributes['shape'] = 'triangle'
            else:
                graphviz_node.attributes['shape'] = 'box'
            graph.append(graphviz_node)
            node_mapping[node] = graphviz_node
            if node.parent is not None:
                graphtools.GraphvizEdge().attach(
                    node_mapping[node.parent],
                    node_mapping[node],
                    )
        return graph

    def __repr__(self):
        r'''Gets interpreter representation of rhythm tree container.

        Returns string.
        '''
        return '{}(({}, {}))'.format(
            type(self).__name__,
            self.duration.numerator,
            self.duration.denominator,
            )

    def __setitem__(self, i, argument):
        r'''Set `argument` in self at nonnegative integer index `i`,
        or set `argument` in self at slice i.
        Replace contents of `self[i]` with `argument`.
        Attach parentage to contents of `argument`,
        and detach parentage of any replaced nodes.

        ::

            >>> a = rhythmtreetools.RhythmTreeContainer()
            >>> b = rhythmtreetools.RhythmTreeLeaf()
            >>> c = rhythmtreetools.RhythmTreeLeaf()

        ::

            >>> a.append(b)
            >>> b.parent is a
            True

        ::

            >>> a.children == (b,)
            True

        ::

            >>> a[0] = c

        ::

            >>> c.parent is a
            True

        ::

            >>> b.parent is None
            True

        ::

            >>> a.children == (c,)
            True

        Return `None`.
        '''
        from abjad.tools.rhythmtreetools.RhythmTreeParser \
            import RhythmTreeParser

        proper_parentage = self.proper_parentage

        if isinstance(i, int):
            if isinstance(argument, str):
                argument = RhythmTreeParser()(argument)[0]
                assert len(argument) == 1
                argument = argument[0]
            else:
                assert isinstance(argument, self._node_class)
            old = self[i]
            assert argument not in proper_parentage
            old._set_parent(None)
            argument._set_parent(self)
            self._children.insert(i, argument)
        else:
            if isinstance(argument, str):
                argument = RhythmTreeParser()(argument)
            elif isinstance(argument, list) and len(argument) == 1 and \
                isinstance(argument[0], str):
                argument = RhythmTreeParser()(argument[0])
            else:
                assert all(isinstance(x, self._node_class) for x in argument)
            if i.start == i.stop and i.start is not None \
                and i.stop is not None and i.start <= -len(self):
                start, stop = 0, 0
            else:
                start, stop, stride = i.indices(len(self))
            old = self[start:stop]
            for node in argument:
                assert node not in proper_parentage
            for node in old:
                node._set_parent(None)
            for node in argument:
                node._set_parent(self)
            self._children.__setitem__(slice(start, start), argument)
        self._mark_entire_tree_for_later_update()

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_duration(self):
        r'''The total preprolated_duration of the children
        of a `RhythmTreeContainer` instance:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree._contents_duration
            Duration(5, 1)

        ::

            >>> tree[1]._contents_duration
            Duration(3, 1)

        Returns int.
        '''
        return sum([x.preprolated_duration for x in self])

    @property
    def _leaf_class(self):
        from abjad.tools import rhythmtreetools
        return rhythmtreetools.RhythmTreeLeaf

    @property
    def _node_class(self):
        return RhythmTreeMixin

    @property
    def _pretty_rtm_format_pieces(self):
        result = []
        result.append('({!s} ('.format(self.preprolated_duration))
        for child in self:
            result.extend(['    ' + x for x in child._pretty_rtm_format_pieces])
        result[-1] = result[-1] + '))'
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def rtm_format(self):
        r'''The node's RTM format:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> tree.rtm_format
            '(1 ((1 (1 1)) (1 (1 1))))'

        Returns string.
        '''
        return '({!s} ({}))'.format(
            self.preprolated_duration,
            ' '.join([x.rtm_format for x in self]))



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Clef(AbjadValueObject):
    r'''Clef.

    ..  container:: example

        **Example 1.** At the beginning of a staff:

        ::

            >>> clef = Clef('treble')
            >>> clef
            Clef(name='treble')

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
            >>> show(staff) # doctest: +SKIP

    ..  container:: example

        **Example 2.** Some available clefs:

        ::

            >>> clef = Clef('treble')
            >>> attach(clef, staff)
            >>> clef = Clef('alto')
            >>> attach(clef, staff[1])
            >>> clef = Clef('bass')
            >>> attach(clef, staff[2])
            >>> clef = Clef('treble^8')
            >>> attach(clef, staff[3])
            >>> clef = Clef('bass_8')
            >>> attach(clef, staff[4])
            >>> clef = Clef('tenor')
            >>> attach(clef, staff[5])
            >>> clef = Clef('bass^15')
            >>> attach(clef, staff[6])
            >>> clef = Clef('percussion')
            >>> attach(clef, staff[7])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \clef "treble"
                c'8
                \clef "alto"
                d'8
                \clef "bass"
                e'8
                \clef "treble^8"
                f'8
                \clef "bass_8"
                g'8
                \clef "tenor"
                a'8
                \clef "bass^15"
                b'8
                \clef "percussion"
                c''8
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_middle_c_position',
        '_name',
        )

    _clef_name_to_middle_c_position = {
        'treble': -6,
        'alto': 0,
        'tenor': 2,
        'bass': 6,
        'french': -8,
        'soprano': -4,
        'mezzosoprano': -2,
        'baritone': 4,
        'varbaritone': 4,
        'percussion': 0,
        'tab': 0,
        }

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, name='treble'):
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        self._default_scope = scoretools.Staff
        if isinstance(name, str):
            self._name = name
        elif isinstance(name, type(self)):
            self._name = name.name
        else:
            message = 'can not initialize clef: {!r}.'
            message = message.format(name)
            raise TypeError(message)
        middle_c_position = self._calculate_middle_c_position(self._name)
        middle_c_position = pitchtools.StaffPosition(middle_c_position)
        self._middle_c_position = middle_c_position

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats clef.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            ::

                >>> clef = Clef('treble')
                >>> print(format(clef))
                indicatortools.Clef(
                    name='treble',
                    )

        ..  container:: example

            ::

                >>> clef = Clef('treble')
                >>> print(format(clef, 'lilypond'))
                \clef "treble"

        Returns string.
        '''
        if format_specification == 'lilypond':
            return self._get_lilypond_format()
        superclass = super(Clef, self)
        return superclass.__format__(format_specification=format_specification)

    def __ne__(self, argument):
        r'''Is true when clef of `argument` does not equal clef name of clef.
        Otherwise false.

        ..  container:: example

            ::

                >>> clef_1 = Clef('treble')
                >>> clef_2 = Clef('alto')

            ::

                >>> clef_1 != clef_1
                False
                >>> clef_1 != clef_2
                True
                >>> clef_2 != clef_1
                True
                >>> clef_2 != clef_2
                False

        Returns true or false.
        '''
        superclass = super(Clef, self)
        return superclass.__ne__(argument)

    ### PRIVATE PROPERTIES ###

    @property
    def _clef_name_to_staff_position_zero(self, clef_name):
        from abjad.tools import pitchtools
        return {
            'treble': pitchtools.NamedPitch('B4'),
            'alto': pitchtools.NamedPitch('C4'),
            'tenor': pitchtools.NamedPitch('A3'),
            'bass': pitchtools.NamedPitch('D3'),
            'french': pitchtools.NamedPitch('D5'),
            'soprano': pitchtools.NamedPitch('G4'),
            'mezzosoprano': pitchtools.NamedPitch('E4'),
            'baritone': pitchtools.NamedPitch('F3'),
            'varbaritone': pitchtools.NamedPitch('F3'),
            'percussion': None,
            'tab': None,
            }[clef_name]

    @property
    def _contents_repr_string(self):
        return repr(self._name)

    ### PRIVATE METHODS ###

    def _calculate_middle_c_position(self, clef_name):
        alteration = 0
        if '_' in self._name:
            base_name, part, suffix = clef_name.partition('_')
            if suffix == '8':
                alteration = 7
            elif suffix == '15':
                alteration = 13
            else:
                message = 'bad clef alteration suffix: {!r}.'
                message = message.format(suffix)
                raise Exception(message)
        elif '^' in self._name:
            base_name, part, suffix = clef_name.partition('^')
            if suffix == '8':
                alteration = -7
            elif suffix == '15':
                alteration = -13
            else:
                message = "bad clef alteration suffix: {!r}."
                message = message.format(suffix)
                raise Exception(message)
        else:
            base_name = clef_name
        return self._clef_name_to_middle_c_position[base_name] + alteration

    def _get_lilypond_format(self):
        return r'\clef "{}"'.format(self._name)

    @classmethod
    def _list_clef_names(cls):
        return list(sorted(cls._clef_name_to_middle_c_position))

    ### PUBLIC METHODS ###

    @staticmethod
    def from_selection(selection):
        r'''Makes clef from `selection`.

        ..  container:: example

            ::

                >>> numbers = list(range(-12, -6))
                >>> notes = scoretools.make_notes(numbers, [Duration(1, 4)])
                >>> staff = Staff(notes)
                >>> Clef.from_selection(staff)
                Clef(name='bass')

            Choses between treble and bass based on minimal number of ledger
            lines.

        Returns new clef.
        '''
        from abjad.tools import pitchtools
        from abjad.tools.topleveltools import iterate
        pitches = list(iterate(selection).by_pitch())
        diatonic_pitch_numbers = [
            pitch.diatonic_pitch_number for pitch in pitches
            ]
        max_diatonic_pitch_number = max(diatonic_pitch_numbers)
        min_diatonic_pitch_number = min(diatonic_pitch_numbers)
        lowest_treble_line_pitch = pitchtools.NamedPitch('e', 4)
        lowest_treble_line_diatonic_pitch_number = \
            lowest_treble_line_pitch.diatonic_pitch_number
        candidate_steps_below_treble = \
            lowest_treble_line_diatonic_pitch_number - \
            min_diatonic_pitch_number
        highest_bass_line_pitch = pitchtools.NamedPitch('a', 3)
        highest_bass_line_diatonic_pitch_number = \
            highest_bass_line_pitch.diatonic_pitch_number
        candidate_steps_above_bass = \
            max_diatonic_pitch_number - highest_bass_line_diatonic_pitch_number
        if candidate_steps_above_bass < candidate_steps_below_treble:
            return Clef('bass')
        else:
            return Clef('treble')

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of clef.

        ..  container:: example

            ::

                >>> clef = Clef('treble')
                >>> clef.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        Clefs are staff-scoped by default.

        Returns staff.
        '''
        return self._default_scope

    @property
    def middle_c_position(self):
        r'''Gets middle C position of clef.

        ..  container:: example

            **Example 1.** Gets staff position of middle C in treble clef:

            ::

                >>> Clef('treble').middle_c_position
                StaffPosition(number=-6)

        ..  container:: example

            **Example 2.** Gets staff position of middle C in alto clef:

            ::

                >>> Clef('alto').middle_c_position
                StaffPosition(number=0)

        Returns nonnegative integer staff position.
        '''
        return self._middle_c_position

    @property
    def name(self):
        r'''Gets name of clef.

        ..  container:: example

            **Example 1.** Gets name treble clef:

            ::

                >>> Clef('treble').name
                'treble'

        ..  container:: example

            **Example 2.** Gets name of alto clef:

            ::

                >>> Clef('alto').name
                'alto'

        Returns string.
        '''
        return self._name



#! /usr/bin/env python
import abjad
import datetime
import subprocess


template = """\
Abjad {abjad_version}
    Build start date: {build_start_date}
    Build complete date:
    MD5 hash of Abjad Github repository: {md5_hash}

    Packaging environment:
        0. workstation description:
        1. uname -v: {kernel_version}
        2. git --version: {git_version}
        3. python --version: {python_version}
        4. pip --version: {pip_version}
        5. py.test --version: {pytest_version}
        6. sphinx-build --version: {sphinx_build_version}
        7. lilypond --version: {lilypond_version}
        8. convert --version: {imagemagick_version}
        9. dot -V: {graphviz_version}

    Python 3.5.2 (OSX) test results:
        1. mainline py.test test results:
        2. mainline doctest test results:
        3. experimental py.test test results:
        4. experimental doctest test results:
        5. sanity check test results:
        6. experimental HTML build results:
        7. mainline HTML build results:

    Python 3.5.2 (Windows) test results:
        1. mainline py.test test results:
        2. mainline doctest test results:
        3. experimental py.test test results:
        4. experimental doctest test results:
        5. sanity check test results:

    Python 2.7.12 (OSX) test results:
        1. mainline py.test test results:
        2. mainline doctest test results:
        3. experimental py.test test results:
        4. experimental doctest test results:
        5. sanity check test results:


"""


def get_abjad_version():
    return abjad.__version__


def get_build_start_date():
    return datetime.date.today().isoformat()


def get_md5_hash():
    command = 'git log -n 1 --pretty=format:"%H"'
    result = subprocess.check_output(command, shell=True).decode('utf-8')
    return result.splitlines()[0]


def get_kernel_version():
    command = 'uname -v'
    result = subprocess.check_output(command, shell=True).decode('utf-8')
    result = result.splitlines()[0]
    return result.partition(':')[0]


def get_git_version():
    command = 'git --version'
    result = subprocess.check_output(command, shell=True).decode('utf-8')
    return result.splitlines()[0]


def get_python_version():
    command = 'python --version'
    pipe = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = pipe.communicate()
    return stdout.decode('utf-8').splitlines()[0]


def get_pip_version():
    command = 'pip -V'
    result = subprocess.check_output(command, shell=True).decode('utf-8')
    return result.partition(' from ')[0]


def get_pytest_version():
    command = 'py.test --version'
    pipe = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    _, stderr = pipe.communicate()
    return stderr.decode('utf-8').splitlines()[0].partition(',')[0]


def get_sphinx_build_version():
    command = 'sphinx-build --version'
    result = subprocess.check_output(command, shell=True).decode('utf-8')
    return result.splitlines()[0]


def get_lilypond_version():
    command = 'lilypond --version'
    result = subprocess.check_output(command, shell=True).decode('utf-8')
    return result.splitlines()[0]


def get_imagemagick_version():
    command = 'convert --version'
    result = subprocess.check_output(command, shell=True).decode('utf-8')
    result = result.splitlines()[0]
    return result.partition('Version: ')[-1]


def get_graphviz_version():
    command = 'dot -V'
    pipe = subprocess.Popen(command, shell=True, stderr=subprocess.PIPE)
    _, stderr = pipe.communicate()
    result = stderr.decode('utf-8').splitlines()[0]
    return result.partition('dot - ')[-1]


abjad_version = get_abjad_version()
build_start_date = get_build_start_date()
md5_hash = get_md5_hash()
kernel_version = get_kernel_version()
git_version = get_git_version()
python_version = get_python_version()
pip_version = get_pip_version()
pytest_version = get_pytest_version()
sphinx_build_version = get_sphinx_build_version()
lilypond_version = get_lilypond_version()
imagemagick_version = get_imagemagick_version()
graphviz_version = get_graphviz_version()


string = template.format(
    abjad_version=abjad_version,
    build_start_date=build_start_date,
    md5_hash=md5_hash,
    kernel_version=kernel_version,
    git_version=git_version,
    python_version=python_version,
    pip_version=pip_version,
    pytest_version=pytest_version,
    sphinx_build_version=sphinx_build_version,
    lilypond_version=lilypond_version,
    imagemagick_version=imagemagick_version,
    graphviz_version=graphviz_version,
    )

print(string)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure_simultaneous_01():
    r'''Measures may be hold simultaneous contents.
    '''

    measure = Measure((2, 8), [])
    measure.append(Voice("c'8 d'8"))
    measure.append(Voice("e'8 f'8"))
    measure.is_simultaneous = True
    command = indicatortools.LilyPondCommand('voiceOne')
    attach(command, measure[0])
    command = indicatortools.LilyPondCommand('voiceTwo')
    attach(command, measure[1])
    staff = Staff([measure])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            <<
                \time 2/8
                \new Voice {
                    \voiceOne
                    c'8
                    d'8
                }
                \new Voice {
                    \voiceTwo
                    e'8
                    f'8
                }
            >>
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_MutationAgent_replace_01():
    r'''Moves parentage and spanners from two old notes to five new notes.

    Equivalent to staff[1:3] = new_notes.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam_1 = Beam()
    attach(beam_1, staff[:2])
    beam_2 = Beam()
    attach(beam_2, staff[2:])
    crescendo = Crescendo()
    attach(crescendo, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ \<
            d'8 ]
            e'8 [
            f'8 ] \!
        }
        '''
        ), format(staff)

    old_notes = staff[1:3]
    new_notes = 5 * Note("c''16")
    mutate(old_notes).replace(new_notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ ] \<
            c''16
            c''16
            c''16
            c''16
            c''16
            f'8 \! [ ]
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_02():
    r'''Moves parentage and spanners from one old note to five new notes.

    Equivalent to staff[:1] = new_notes.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam_1 = Beam()
    attach(beam_1, staff[:2])
    beam_2 = Beam()
    attach(beam_2, staff[2:])
    crescendo = Crescendo()
    attach(crescendo, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ \<
            d'8 ]
            e'8 [
            f'8 ] \!
        }
        '''
        ), format(staff)

    old_notes = staff[:1]
    new_notes = 5 * Note("c''16")
    mutate(old_notes).replace(new_notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c''16 [ \<
            c''16
            c''16
            c''16
            c''16
            d'8 ]
            e'8 [
            f'8 ] \!
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_03():
    r'''Moves parentage and spanners from two old notes to five new notes.

    Equivalent to staff[:2] = new_notes.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam_1 = Beam()
    attach(beam_1, staff[:2])
    beam_2 = Beam()
    attach(beam_2, staff[2:])
    crescendo = Crescendo()
    attach(crescendo, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ \<
            d'8 ]
            e'8 [
            f'8 ] \!
        }
        '''
        ), format(staff)

    old_notes = staff[:2]
    new_notes = 5 * Note("c''16")
    mutate(old_notes).replace(new_notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c''16 [ \<
            c''16
            c''16
            c''16
            c''16 ]
            e'8 [
            f'8 ] \!
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_04():
    r'''Moves parentage and spanners from three old notes to five new notes.

    "Equivalent to staff[:3] = new_notes."
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam_1 = Beam()
    attach(beam_1, staff[:2])
    beam_2 = Beam()
    attach(beam_2, staff[2:])
    crescendo = Crescendo()
    attach(crescendo, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ \<
            d'8 ]
            e'8 [
            f'8 ] \!
        }
        '''
        ), format(staff)

    old_notes = staff[:3]
    new_notes = 5 * Note("c''16")
    mutate(old_notes).replace(new_notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c''16 \<
            c''16
            c''16
            c''16
            c''16
            f'8 \! [ ]
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_05():
    r'''Moves parentage and spanners from four old notes to five new notes.

    Equivalent to staff[:] = new_notes.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam_1 = Beam()
    attach(beam_1, staff[:2])
    beam_2 = Beam()
    attach(beam_2, staff[2:])
    crescendo = Crescendo()
    attach(crescendo, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ \<
            d'8 ]
            e'8 [
            f'8 ] \!
        }
        '''
        ), format(staff)

    old_notes = staff[:]
    new_notes = 5 * Note("c''16")
    mutate(old_notes).replace(new_notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c''16 \<
            c''16
            c''16
            c''16
            c''16 \!
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_06():
    r'''Moves parentage and spanners from container to children of container.

    Replaces container with contents of container.

    Effectively removes container from score.

    Equivalent to staff[:1] = staff[0][:].
    '''

    staff = Staff([Voice("c'8 d'8 e'8 f'8")])
    beam = Beam()
    attach(beam, staff[0][:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8 [
                d'8
                e'8
                f'8 ]
            }
        }
        '''
        ), format(staff)

    voice_selection = staff[:1]
    voice = voice_selection[0]
    old_components = mutate(voice_selection).replace(staff[0][:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        ), format(staff)

    assert not voice
    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_UnweightedSearchTree__find_divisible_leaf_indices_and_subdivisions_01():

    definition = {
        2: {
            2: {
                2: None
            },
            3: None
        },
        5: None
    }
    search_tree = quantizationtools.UnweightedSearchTree(definition)

    q_grid = quantizationtools.QGrid()
    a = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(0,      ['A']), 0, 1)
    b = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 5), ['B']), 0, 1)
    c = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 4), ['C']), 0, 1)
    d = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 3), ['D']), 0, 1)
    e = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 5), ['E']), 0, 1)
    f = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((1, 2), ['F']), 0, 1)
    g = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 5), ['G']), 0, 1)
    h = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((2, 3), ['H']), 0, 1)
    i = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((3, 4), ['I']), 0, 1)
    j = quantizationtools.QEventProxy(quantizationtools.SilentQEvent((4, 5), ['J']), 0, 1)
    k = quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1,      ['K']), 0, 1)
    q_grid.fit_q_events([a, b, c, d, e, f, g, h, i, j, k])

    indices, subdivisions = search_tree._find_divisible_leaf_indices_and_subdivisions(q_grid)

    assert indices == [0]
    assert subdivisions == [((1, 1), (1, 1, 1, 1, 1))]



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_PhrasingSlur___init___01():
    r'''Initialize empty phrasing slur.
    '''

    phrasing_slur = spannertools.PhrasingSlur()
    assert isinstance(phrasing_slur, spannertools.PhrasingSlur)


def test_spannertools_PhrasingSlur___init___02():

    staff = Staff("c'8 d'8 e'8 f'8")
    phrasing_slur = spannertools.PhrasingSlur()
    attach(phrasing_slur, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \(
            d'8
            e'8
            f'8 \)
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- coding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_DateTimeToken___format___01():

    date_time_token = lilypondfiletools.DateTimeToken()
    assert isinstance(date_time_token._get_lilypond_format(), str)
    assert len(date_time_token._get_lilypond_format()) == 16



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.pitchtools import NamedPitchClass


def test_pitchtools_NamedPitchClass___repr___01():
    r'''Named pitch-class repr is evaluable.
    '''

    named_pitch_class_1 = pitchtools.NamedPitchClass('cs')
    named_pitch_class_2 = eval(repr(named_pitch_class_1))

    "NamedPitchClass('cs')"

    assert isinstance(named_pitch_class_1, pitchtools.NamedPitchClass)
    assert isinstance(named_pitch_class_2, pitchtools.NamedPitchClass)



# -*- coding: utf-8 -*-
import copy
import hashlib
import platform
import os
import subprocess
import sys
from abjad.tools import systemtools
from abjad.tools.abjadbooktools.ImageOutputProxy import ImageOutputProxy


class GraphvizOutputProxy(ImageOutputProxy):
    r'''A Graphviz output proxy.

    ::

        >>> from abjad.tools import abjadbooktools
        >>> meter = metertools.Meter((4, 4))
        >>> proxy = abjadbooktools.GraphvizOutputProxy(meter)
        >>> print(format(proxy))
        ...
        abjadbooktools.GraphvizOutputProxy(
            graphtools.GraphvizGraph(
                attributes={
                    'bgcolor': 'transparent',
                    'fontname': 'Arial',
                    'penwidth': 2,
                    'truecolor': True,
                    },
                children=(
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '4/4',
                            'shape': 'triangle',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '1/4',
                            'shape': 'box',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '1/4',
                            'shape': 'box',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '1/4',
                            'shape': 'box',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '1/4',
                            'shape': 'box',
                            },
                        ),
                    graphtools.GraphvizSubgraph(
                        attributes={
                            'style': 'rounded',
                            },
                        children=(
                            graphtools.GraphvizNode(
                                attributes={
                                    'color': 'white',
                                    'fillcolor': 'black',
                                    'fontcolor': 'white',
                                    'fontname': 'Arial bold',
                                    'shape': 'Mrecord',
                                    'style': 'filled',
                                    },
                                children=(
                                    graphtools.GraphvizGroup(
                                        children=(
                                            graphtools.GraphvizField(
                                                label='0',
                                                ),
                                            graphtools.GraphvizField(
                                                label='++',
                                                ),
                                            ),
                                        ),
                                    ),
                                ),
                            graphtools.GraphvizNode(
                                attributes={
                                    'color': 'white',
                                    'fillcolor': 'black',
                                    'fontcolor': 'white',
                                    'fontname': 'Arial bold',
                                    'shape': 'Mrecord',
                                    'style': 'filled',
                                    },
                                children=(
                                    graphtools.GraphvizGroup(
                                        children=(
                                            graphtools.GraphvizField(
                                                label='1/4',
                                                ),
                                            graphtools.GraphvizField(
                                                label='+',
                                                ),
                                            ),
                                        ),
                                    ),
                                ),
                            graphtools.GraphvizNode(
                                attributes={
                                    'color': 'white',
                                    'fillcolor': 'black',
                                    'fontcolor': 'white',
                                    'fontname': 'Arial bold',
                                    'shape': 'Mrecord',
                                    'style': 'filled',
                                    },
                                children=(
                                    graphtools.GraphvizGroup(
                                        children=(
                                            graphtools.GraphvizField(
                                                label='1/2',
                                                ),
                                            graphtools.GraphvizField(
                                                label='+',
                                                ),
                                            ),
                                        ),
                                    ),
                                ),
                            graphtools.GraphvizNode(
                                attributes={
                                    'color': 'white',
                                    'fillcolor': 'black',
                                    'fontcolor': 'white',
                                    'fontname': 'Arial bold',
                                    'shape': 'Mrecord',
                                    'style': 'filled',
                                    },
                                children=(
                                    graphtools.GraphvizGroup(
                                        children=(
                                            graphtools.GraphvizField(
                                                label='3/4',
                                                ),
                                            graphtools.GraphvizField(
                                                label='+',
                                                ),
                                            ),
                                        ),
                                    ),
                                ),
                            graphtools.GraphvizNode(
                                attributes={
                                    'shape': 'Mrecord',
                                    },
                                children=(
                                    graphtools.GraphvizGroup(
                                        children=(
                                            graphtools.GraphvizField(
                                                label='1',
                                                ),
                                            graphtools.GraphvizField(
                                                label='++',
                                                ),
                                            ),
                                        ),
                                    ),
                                ),
                            ),
                        edge_attributes={
                            },
                        is_cluster=True,
                        name='cluster_offsets',
                        node_attributes={
                            },
                        ),
                    ),
                edge_attributes={
                    'penwidth': 2,
                    },
                is_digraph=True,
                name='G',
                node_attributes={
                    'fontname': 'Arial',
                    'fontsize': 12,
                    'penwidth': 2,
                    },
                ),
            layout='dot',
            )

    ::

        >>> proxy.as_latex(relative_output_directory='assets')
        ['\\noindent\\includegraphics{assets/graphviz-31410f5aefd17473e91ebc219ddff36e.pdf}']

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Output Proxies'

    __slots__ = (
        '_layout',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        payload,
        layout='dot',
        image_layout_specifier=None,
        image_render_specifier=None,
        ):
        ImageOutputProxy.__init__(
            self,
            image_layout_specifier=image_layout_specifier,
            image_render_specifier=image_render_specifier,
            )
        graphviz_graph = copy.deepcopy(payload.__graph__())
        self._payload = graphviz_graph
        self._layout = layout

    ### PRIVATE METHODS ###

    def _render_pdf_source(
        self,
        temporary_directory_path,
        ):
        dot_file_path = os.path.join(
            temporary_directory_path,
            self.file_name_without_extension + '.dot',
            )
        source = str(self.payload)
        with open(dot_file_path, 'w') as file_pointer:
            file_pointer.write(source)
        pdf_file_path = os.path.join(
            temporary_directory_path,
            self.file_name_without_extension + '.pdf',
            )
        if platform.system() == 'Darwin':
            command = '{} -v -Tpdf:quartz:quartz {} -o {}'
        else:
            command = '{} -v -Tpdf {} -o {}'
        command = command.format(
            self.layout,
            dot_file_path,
            pdf_file_path,
            )
        exit_code = subprocess.call(
            command,
            shell=True,
            #stdout=subprocess.PIPE,
            #stderr=subprocess.PIPE,
            )
        if exit_code:
            print(source)
            raise AssertionError
        assert os.path.exists(pdf_file_path)
        assert systemtools.IOManager.find_executable('pdfcrop')
        command = 'pdfcrop {path} {path}'.format(path=pdf_file_path)
        exit_code = subprocess.call(
            command,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            )
        assert exit_code == 0
        return pdf_file_path

    ### PUBLIC METHODS ###

    def as_docutils(
        self,
        configuration=None,
        output_directory=None,
        ):
        r'''Creates a docutils node representation of the output proxy.

        ::

            >>> for node in proxy.as_docutils():
            ...     print(node.pformat())
            ...
            <abjad_output_block image_layout_specifier... image_render_specifier... layout="dot" renderer="graphviz" xml:space="preserve">
                digraph G {
                    graph [bgcolor=transparent,
                        fontname=Arial,
                        penwidth=2,
                        truecolor=true];
                    node [fontname=Arial,
                        fontsize=12,
                        penwidth=2];
                    edge [penwidth=2];
                    node_0 [label="4/4",
                        shape=triangle];
                    node_1 [label="1/4",
                        shape=box];
                    node_2 [label="1/4",
                        shape=box];
                    node_3 [label="1/4",
                        shape=box];
                    node_4 [label="1/4",
                        shape=box];
                    subgraph cluster_cluster_offsets {
                        graph [style=rounded];
                        node_5_0 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 0 | <f_0_1> ++ }",
                            shape=Mrecord,
                            style=filled];
                        node_5_1 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 1/4 | <f_0_1> + }",
                            shape=Mrecord,
                            style=filled];
                        node_5_2 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 1/2 | <f_0_1> + }",
                            shape=Mrecord,
                            style=filled];
                        node_5_3 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 3/4 | <f_0_1> + }",
                            shape=Mrecord,
                            style=filled];
                        node_5_4 [label="{ <f_0_0> 1 | <f_0_1> ++ }",
                            shape=Mrecord];
                    }
                    node_0 -> node_1;
                    node_0 -> node_2;
                    node_0 -> node_3;
                    node_0 -> node_4;
                    node_1 -> node_5_0 [style=dotted];
                    node_1 -> node_5_1 [style=dotted];
                    node_2 -> node_5_1 [style=dotted];
                    node_2 -> node_5_2 [style=dotted];
                    node_3 -> node_5_2 [style=dotted];
                    node_3 -> node_5_3 [style=dotted];
                    node_4 -> node_5_3 [style=dotted];
                    node_4 -> node_5_4 [style=dotted];
                }
            <BLANKLINE>

        Returns list of docutils nodes.
        '''
        from abjad.tools import abjadbooktools
        result = []
        try:
            code = str(self.payload)
            if sys.version_info[0] == 2:
                code = code.decode('utf-8')
            node = abjadbooktools.abjad_output_block(code, code)
            node['image_layout_specifier'] = self.image_layout_specifier
            node['image_render_specifier'] = self.image_render_specifier
            node['layout'] = self.layout
            node['renderer'] = 'graphviz'
            result.append(node)
        except UnicodeDecodeError:
            print()
            print(type(self))
            for line in code.splitlines():
                print(repr(line))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def file_name_prefix(self):
        r'''Gets file name prefix of Graphviz output proxy.

        Returns string.
        '''
        return 'graphviz'

    @property
    def file_name_without_extension(self):
        r'''Gets file name extension of Graphviz output proxy.

        Returns string.
        '''
        payload = '\n'.join(str(self.payload))
        md5 = hashlib.md5(payload.encode()).hexdigest()
        return '-'.join((self.file_name_prefix, md5))

    @property
    def layout(self):
        r'''Gets layout engine name of Graphviz output.

        Returns string.
        '''
        return self._layout



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Staff___getitem___01():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    assert isinstance(staff[0], Note)
    assert isinstance(staff[1], Rest)
    assert isinstance(staff[2], Chord)
    assert isinstance(staff[3], scoretools.Skip)
    assert isinstance(staff[4], scoretools.FixedDurationTuplet)
    assert isinstance(staff[-5], Note)
    assert isinstance(staff[-4], Rest)
    assert isinstance(staff[-3], Chord)
    assert isinstance(staff[-2], scoretools.Skip)
    assert isinstance(staff[-1], scoretools.FixedDurationTuplet)


def test_scoretools_Staff___getitem___02():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    slice = staff[0:0]
    assert len(slice) == 0
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___getitem___03():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    slice = staff[0:1]
    assert len(slice) == 1
    assert isinstance(slice[0], Note)
    for x in staff:
        assert x._parent == staff
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___getitem___04():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    slice = staff[-1:]
    assert len(slice) == 1
    assert isinstance(slice[0], scoretools.FixedDurationTuplet)
    for x in slice:
        assert x._parent == staff
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___getitem___05():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    slice = staff[1:-1]
    assert len(slice) == 3
    assert isinstance(slice[0], Rest)
    assert isinstance(slice[1], Chord)
    assert isinstance(slice[2], scoretools.Skip)
    for x in slice:
        assert x._parent == staff
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___getitem___06():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    slice = staff[2:]
    assert len(slice) == 3
    assert isinstance(slice[0], Chord)
    assert isinstance(slice[1], scoretools.Skip)
    assert isinstance(slice[2], scoretools.FixedDurationTuplet)
    for x in slice:
        assert x._parent == staff
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___getitem___07():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    slice = staff[:-2]
    assert len(slice) == 3
    assert isinstance(slice[0], Note)
    assert isinstance(slice[1], Rest)
    assert isinstance(slice[2], Chord)
    for x in slice:
        assert x._parent == staff
    assert inspect_(staff).is_well_formed()


def test_scoretools_Staff___getitem___08():
    staff = Staff([Note("c'4"),
            Rest((1, 4)),
            Chord([2, 3, 4], (1, 4)),
            scoretools.Skip((1, 4)),
            scoretools.FixedDurationTuplet(Duration(5, 16), Note(0, (1, 16)) * 4)])
    assert len(staff) == 5
    assert inspect_(staff).is_well_formed()
    slice = staff[:]
    assert len(slice) == 5
    assert isinstance(slice, (list, selectiontools.Selection))
    assert isinstance(slice[0], Note)
    assert isinstance(slice[1], Rest)
    assert isinstance(slice[2], Chord)
    assert isinstance(slice[3], scoretools.Skip)
    assert isinstance(slice[4], scoretools.FixedDurationTuplet)
    for x in slice:
        assert x._parent == staff
    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
import abc
from abjad import Fraction
from abjad.tools import abctools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools


class RhythmTreeMixin(abctools.AbjadObject):
    r'''Rhythm-tree node abstract base class.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, preprolated_duration=1):
        self._duration = 0
        self._offset = durationtools.Offset(0)
        self._offsets_are_current = False
        self.preprolated_duration = preprolated_duration

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, pulse_duration):
        r'''Calls rhythm tree node on `pulse_duration`.
        '''
        raise NotImplementedError

    ### PRIVATE METHODS ###

    def _update_offsets_of_entire_tree(self):
        def recurse(container, current_offset):
            container._offset = current_offset
            container._offsets_are_current = True
            for child in container:
                if hasattr(child, 'children'):
                    current_offset = recurse(child, current_offset)
                else:
                    child._offset = current_offset
                    child._offsets_are_current = True
                    current_offset += child.duration
            return current_offset
        offset = durationtools.Offset(0)
        root = self.root
        if root is self and not hasattr(self, 'children'):
            self._offset = offset
            self._offsets_are_current = True
        else:
            recurse(root, offset)

    def _update_offsets_of_entire_tree_if_necessary(self):
        if not self._get_node_state_flags()['_offsets_are_current']:
            self._update_offsets_of_entire_tree()

    ### PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _pretty_rtm_format_pieces(self):
        raise NotImplementedError

    @property
    def _state_flag_names(self):
        return ('_offsets_are_current',)

    ### PUBLIC PROPERTIES ###

    @property
    def duration(self):
        r'''The preprolated_duration of the node:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree.duration
            Duration(1, 1)

        ::

            >>> tree[1].duration
            Duration(1, 2)

        ::

            >>> tree[1][1].duration
            Duration(1, 4)

        Return `Duration` instance.
        '''
        return self.prolation * self.preprolated_duration

    @property
    def parentage_ratios(self):
        r'''A sequence describing the relative durations of the nodes in a
        node's improper parentage.

        The first item in the sequence is the preprolated_duration of
        the root node, and subsequent items are pairs of the
        preprolated duration of the next node in the parentage and
        the total preprolated_duration of that node and its siblings:


        ::

            >>> a = rhythmtreetools.RhythmTreeContainer(preprolated_duration=1)
            >>> b = rhythmtreetools.RhythmTreeContainer(preprolated_duration=2)
            >>> c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
            >>> d = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=4)
            >>> e = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=5)

        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])

        ::

            >>> a.parentage_ratios
            (Duration(1, 1),)

        ::

            >>> b.parentage_ratios
            (Duration(1, 1), (Duration(2, 1), Duration(5, 1)))

        ::

            >>> c.parentage_ratios
            (Duration(1, 1), (Duration(3, 1), Duration(5, 1)))

        ::

            >>> d.parentage_ratios
            (Duration(1, 1), (Duration(2, 1), Duration(5, 1)), (Duration(4, 1), Duration(9, 1)))

        ::

            >>> e.parentage_ratios
            (Duration(1, 1), (Duration(2, 1), Duration(5, 1)), (Duration(5, 1), Duration(9, 1)))

        Returns tuple.
        '''
        result = []
        node = self
        while node.parent is not None:
            result.append(
                (node.preprolated_duration, node.parent._contents_duration))
            node = node.parent
        result.append(node.preprolated_duration)
        return tuple(reversed(result))

    @property
    def preprolated_duration(self):
        r'''The node's preprolated_duration in pulses:

        ::

            >>> node = rhythmtreetools.RhythmTreeLeaf(
            ...     preprolated_duration=1)
            >>> node.preprolated_duration
            Duration(1, 1)

        ::

            >>> node.preprolated_duration = 2
            >>> node.preprolated_duration
            Duration(2, 1)

        Returns int.
        '''
        return self._duration

    @preprolated_duration.setter
    def preprolated_duration(self, arg):
        if not isinstance(arg, Fraction):
            arg = durationtools.Duration(arg)
        assert 0 < arg
        self._duration = arg
        self._mark_entire_tree_for_later_update()

    @property
    def pretty_rtm_format(self):
        r'''The node's pretty-printed RTM format:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> print(tree.pretty_rtm_format)
            (1 (
                (1 (
                    1
                    1))
                (1 (
                    1
                    1))))

        Returns string.
        '''
        return '\n'.join(self._pretty_rtm_format_pieces)

    @property
    def prolation(self):
        r'''Prolation of rhythm tree node.

        Returns multiplier.
        '''
        return mathtools.cumulative_products(self.prolations)[-1]

    @property
    def prolations(self):
        r'''Prolations of rhythm tree node.

        Returns tuple.
        '''
        prolations = [durationtools.Multiplier(1)]
        improper_parentage = self.improper_parentage
        pairs = sequencetools.Sequence(improper_parentage).nwise()
        for child, parent in pairs:
            prolations.append(durationtools.Multiplier(
                parent.preprolated_duration, parent._contents_duration))
        return tuple(prolations)

    @abc.abstractproperty
    def rtm_format(self):
        r'''The node's RTM format:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> tree.rtm_format
            '(1 ((1 (1 1)) (1 (1 1))))'

        Returns string.
        '''
        raise NotImplementedError

    @property
    def start_offset(self):
        r'''The starting offset of a node in a rhythm-tree relative the root.

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree.start_offset
            Offset(0, 1)

        ::

            >>> tree[1].start_offset
            Offset(1, 2)

        ::

            >>> tree[0][1].start_offset
            Offset(1, 4)

        Returns Offset instance.
        '''
        self._update_offsets_of_entire_tree_if_necessary()
        return self._offset

    @property
    def stop_offset(self):
        r'''The stopping offset of a node in a rhythm-tree relative the root.
        '''
        return self.start_offset + self.duration



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Decrescendo_direction_01():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    decrescendo = Decrescendo(direction=Up)
    attach(decrescendo, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 ^ \>
            d'8
            e'8
            f'8 \!
            g'2
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.indicatortools import LilyPondCommand


def test_indicatortools_LilyPondCommand___repr___01():
    r'''Repr of unattached LilyPond command is evaluable.
    '''

    lily_pond_command_1 = indicatortools.LilyPondCommand('break')
    lily_pond_command_2 = eval(repr(lily_pond_command_1))

    assert isinstance(lily_pond_command_1, indicatortools.LilyPondCommand)
    assert isinstance(lily_pond_command_2, indicatortools.LilyPondCommand)
    assert lily_pond_command_1 == lily_pond_command_2



# -*- coding: utf-8 -*-
import inspect
import pytest
from abjad.tools import documentationtools
from abjad.tools import systemtools


ignored_classes = (
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.skip('Correct failing classes.')
@pytest.mark.parametrize('class_', classes)
def test_abjad___radd___01(class_):
    r'''All classes implementing __add__ also implement __radd__.
    '''
    if inspect.isabstract(class_):
        return
    if hasattr(class_, '__add__'):
        assert hasattr(class_, '__radd__')



from abjad import *


def test_agenttools_InspectionAgent_get_annotation_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    annotation = indicatortools.Annotation('special dictionary', {})
    attach(annotation, staff[0])

    assert inspect_(staff[0]).get_annotation('special dictionary') == {}



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitchClass_pitch_class_name_01():

    assert pitchtools.NamedPitchClass("c''").pitch_class_name == 'c'
    assert pitchtools.NamedPitchClass("d''").pitch_class_name == 'd'
    assert pitchtools.NamedPitchClass("e''").pitch_class_name == 'e'
    assert pitchtools.NamedPitchClass("f''").pitch_class_name == 'f'



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NamedPitchClass___slots___01():

    named_pitch_class = pitchtools.NamedPitchClass("cs")
    assert pytest.raises(AttributeError, "named_pitch_class.foo = 'bar'")



# -*- coding: utf-8 -*-


def pluralize(string, count=None):
    r'''Pluralizes English `string`.
    
    ..  container:: example

        Changes terminal `-y` to `-ies`:

        ::

            >>> stringtools.pluralize('catenary')
            'catenaries'

    ..  container:: example

        Adds `-es` to terminal `-s`, `-sh`, `-x` and `-z`:

        ::

            >>> stringtools.pluralize('brush')
            'brushes'

    ..  container:: example

        Adds `-s` to all other strings:

        ::

            >>> stringtools.pluralize('shape')
            'shapes'

    Returns string.
    '''
    if count == 1:
        return string
    if string.endswith('y'):
        return string[:-1] + 'ies'
    elif string.endswith(('s', 'sh', 'x', 'z')):
        return string + 'es'
    else:
        return string + 's'



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Chord___ne___01():

    chord_1 = Chord("<c' e' fs'>4")
    chord_2 = Chord("<c' e' fs'>4")
    chord_3 = Chord("<c' e' fs'>4")

    assert not chord_1 != chord_1
    assert     chord_1 != chord_2
    assert     chord_1 != chord_3
    assert     chord_2 != chord_1
    assert not chord_2 != chord_2
    assert     chord_2 != chord_3
    assert     chord_3 != chord_1
    assert     chord_3 != chord_2
    assert not chord_3 != chord_3



# -*- coding: utf-8 -*-
import six


def delimit_words(string):
    r'''Delimits words in `string`.

    ..  container:: example

        Delimits words::

            >>> stringtools.delimit_words('scale degrees 4 and 5.')
            ['scale', 'degrees', '4', 'and', '5']

    ..  container:: example

        Delimits conjoined words::

            >>> stringtools.delimit_words('scale degrees 4and5.')
            ['scale', 'degrees', '4', 'and', '5']

    ..  container:: example

        Delimits lower camel case::

            >>> stringtools.delimit_words('scaleDegrees4and5.')
            ['scale', 'Degrees', '4', 'and', '5']

    ..  container:: example

        Delimits upper camel case::

            >>> stringtools.delimit_words('ScaleDegrees4and 5.')
            ['Scale', 'Degrees', '4', 'and', '5']

    ..  container:: example

        Delimits dash case::

            >>> stringtools.delimit_words('scale-degrees-4-and-5.')
            ['scale', 'degrees', '4', 'and', '5']

    ..  container:: example

        Delimits shout case::

            >>> stringtools.delimit_words('SCALE_DEGREES_4_AND_5.')
            ['SCALE', 'DEGREES', '4', 'AND', '5']

    ..  container:: example

        Works with greater-than and less-than signs:

            >>> stringtools.delimit_words('one < two')
            ['one', '<', 'two']

    ..  container:: example

        Works with exclamation points:

            >>> stringtools.delimit_words('one! two!')
            ['one', '!', 'two', '!']

    Returns list.
    '''
    assert isinstance(string, six.string_types), repr(string)
    wordlike_characters = ('<', '>', '!')
    words = []
    current_word = ''
    for character in string:
        if (not character.isalpha() and
            not character.isdigit() and
            not character in wordlike_characters
            ):
            if current_word:
                words.append(current_word)
                current_word = ''
        elif not current_word:
            current_word = current_word + character
        elif character.isupper():
            if current_word[-1].isupper():
                current_word = current_word + character
            else:
                words.append(current_word)
                current_word = character
        elif character.islower():
            if current_word[-1].isalpha():
                current_word = current_word + character
            else:
                words.append(current_word)
                current_word = character
        elif character.isdigit():
            if current_word[-1].isdigit():
                current_word = current_word + character
            else:
                words.append(current_word)
                current_word = character
        elif character in wordlike_characters:
            if current_word[-1] in wordlike_characters:
                current_word = current_word + character
            else:
                words.append(current_word)
                current_word = character
    if current_word:
        words.append(current_word)
    return words



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.topleveltools import iterate


def notes_and_chords_are_in_range(argument):
    '''Is true when notes and chords in `argument` are within traditional
    instrument ranges.

    ..  todo:: Move to WellformednessManager.

    ..  container:: example

        **Example 1.** In range:

        ::

            >>> staff = Staff("c'8 r8 <d' fs'>8 r8")
            >>> violin = instrumenttools.Violin()
            >>> attach(violin, staff)
            >>> show(staff) # doctest: +SKIP

        ::

            >>> instrumenttools.notes_and_chords_are_in_range(staff)
            True

    ..  container:: example

        **Example 2.** Out of range:

        ::

            >>> staff = Staff("c'8 r8 <d fs>8 r8")
            >>> violin = instrumenttools.Violin()
            >>> attach(violin, staff)
            >>> show(staff) # doctest: +SKIP

        ::

            >>> instrumenttools.notes_and_chords_are_in_range(staff)
            False

    Returns true or false.
    '''
    from abjad.tools import instrumenttools
    prototype = (scoretools.Note, scoretools.Chord)
    for note_or_chord in iterate(argument).by_class(prototype):
        instrument = note_or_chord._get_effective(instrumenttools.Instrument)
        if note_or_chord not in instrument.pitch_range:
            return False
    else:
        return True


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Note___setattr___01():
    r'''Slots constrain note attributes.
    '''

    note = Note("c'4")

    assert pytest.raises(AttributeError, "note.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_analyze_passing_tones_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    selection = tonalanalysistools.select(staff[:])

    result = [False, True, True, False]
    assert selection.analyze_passing_tones() == result



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitch_pitch_number_01():

    assert pitchtools.NumberedPitch("cff''").pitch_number == 10
    assert pitchtools.NumberedPitch("ctqf''").pitch_number == 10.5
    assert pitchtools.NumberedPitch("cf''").pitch_number == 11
    assert pitchtools.NumberedPitch("cqf''").pitch_number == 11.5
    assert pitchtools.NumberedPitch("c''").pitch_number == 12
    assert pitchtools.NumberedPitch("cqs''").pitch_number == 12.5
    assert pitchtools.NumberedPitch("cs''").pitch_number == 13
    assert pitchtools.NumberedPitch("ctqs''").pitch_number == 13.5
    assert pitchtools.NumberedPitch("css''").pitch_number == 14
    assert pitchtools.NumberedPitch("d''").pitch_number == 14



# -*- coding: utf-8 -*-
import abc
import re
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class Interval(AbjadValueObject):
    '''Interval base class.
    '''

    ### CLASS VARIABLES ###

    _named_interval_quality_abbreviation_regex_body = '''
        (M|         # major
        m|          # minor
        P|          # perfect
        aug|        # augmented
        dim)        # dimished
        '''

    _named_interval_quality_abbreviation_regex = re.compile(
        '^{}$'.format(_named_interval_quality_abbreviation_regex_body),
        re.VERBOSE,
        )

    _interval_name_abbreviation_regex_body = '''
        ([+,-]?)    # one plus, one minus, or neither
        {}          # exactly one quality abbreviation
        (\d+)       # followed by one or more digits
        '''.format(
        _named_interval_quality_abbreviation_regex_body,
        )

    _interval_name_abbreviation_regex = re.compile(
        '^{}$'.format(_interval_name_abbreviation_regex_body),
        re.VERBOSE,
        )

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of interval.

        Returns new interval.
        '''
        return type(self)(abs(self.number))

    def __float__(self):
        r'''Change interval to float.

        Returns float.
        '''
        message = 'float needs to be implemented on {}.'
        message = message.format(type(self))
        raise NotImplementedError(message)

    def __int__(self):
        r'''Change interval to integer.

        Returns integer.
        '''
        message = 'int needs to be implemented on {}.'
        message = message.format(type(self))
        raise NotImplementedError(message)

    def __ne__(self, argument):
        r'''Is true when interval does not equal `argument`.

        Returns true or false.
        '''
        return not self == argument

    def __neg__(self):
        r'''Negates interval.

        Returns interval.
        '''
        pass

    def __str__(self):
        r'''String representation of interval.

        Returns string.
        '''
        return str(self.number)

    ### PRIVATE PROPERTIES ###

    @property
    def _direction_symbol(self):
        if self.direction_number == -1:
            return '-'
        elif self.direction_number == 0:
            return ''
        elif self.direction_number == 1:
            return '+'
        else:
            raise ValueError

    @property
    def _format_string(self):
        return str(self.number)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        if type(self).__name__.startswith('Named'):
            values = [str(self)]
        else:
            values = [
                mathtools.integer_equivalent_number_to_integer(float(self))
                ]
        return systemtools.FormatSpecification(
            client=self,
            coerce_for_equality=True,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            template_names=['direction_number', 'interval_number'],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def cents(self):
        r'''Cents of interval.

        Returns nonnegative number.
        '''
        return 100 * self.semitones

    ### PUBLIC METHODS ###

    @staticmethod
    def is_named_interval_abbreviation(argument):
        '''Is true when `argument` is a named interval abbreviation.
        Otherwise false:

        ::

            >>> pitchtools.Interval.is_named_interval_abbreviation('+M9')
            True

        The regex ``^([+,-]?)(M|m|P|aug|dim)(\d+)$`` underlies this predicate.

        Returns true or false.
        '''
        if not isinstance(argument, str):
            return False
        return bool(Interval._interval_name_abbreviation_regex.match(argument))

    @staticmethod
    def is_named_interval_quality_abbreviation(argument):
        '''Is true when `argument` is a named-interval quality abbreviation. Otherwise
        false:

        ::

            >>> pitchtools.Interval.is_named_interval_quality_abbreviation('aug')
            True

        The regex ``^M|m|P|aug|dim$`` underlies this predicate.

        Returns true or false.
        '''
        if not isinstance(argument, str):
            return False
        return bool(Interval._named_interval_quality_abbreviation_regex.match(
            argument))



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Measure_empty_01():

    measure = Measure((4, 4), [])

    assert str(measure) == 'Measure((4, 4))'
    assert pytest.raises(UnderfullContainerError, 'format(measure)')
    assert len(measure) == 0
    assert measure._preprolated_duration == 0
    assert inspect_(measure).get_duration() == 0
    assert not inspect_(measure).is_well_formed()


def test_scoretools_Measure_empty_02():

    measure = Measure((4, 5), [])

    assert str(measure) == 'Measure((4, 5))'
    assert pytest.raises(UnderfullContainerError, 'format(measure)')
    assert len(measure) == 0
    assert measure._preprolated_duration == 0
    assert inspect_(measure).get_duration() == 0
    assert not inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner_name_01():
    staff = Staff("c'4 d'4 e'4 f'4")
    spanner = spannertools.Spanner()
    assert spanner.name is None
    attach(spanner, staff[:])
    assert spanner.name is None


def test_spannertools_Spanner_name_02():
    staff = Staff("c'4 d'4 e'4 f'4")
    spanner = spannertools.Spanner(name='foo')
    assert spanner.name == 'foo'
    attach(spanner, staff[:])
    assert spanner.name == 'foo'


def test_spannertools_Spanner_name_03():
    staff = Staff("c'4 d'4 e'4 f'4")
    spanner = spannertools.Spanner(name='foo')
    assert spanner.name == 'foo'
    attach(spanner, staff[:], name='bar')
    assert spanner.name == 'bar'


def test_spannertools_Spanner_name_04():
    staff = Staff("c'4 d'4 e'4 f'4")
    spanner = spannertools.Spanner()
    assert spanner.name is None
    attach(spanner, staff[:], name='bar')
    assert spanner.name == 'bar'



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection__get_offset_lists_01():

    staff = Staff("c'4 d'4 e'4 f'4")
    selection = staff[:2]
    start_offsets, stop_offsets = selection._get_offset_lists()

    assert start_offsets == [Offset(0, 1), Offset(1, 4)]
    assert stop_offsets == [Offset(1, 4), Offset(1, 2)]



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClass_is_pitch_class_number_01():

    assert pitchtools.PitchClass.is_pitch_class_number(0)
    assert pitchtools.PitchClass.is_pitch_class_number(0.5)
    assert pitchtools.PitchClass.is_pitch_class_number(11)
    assert pitchtools.PitchClass.is_pitch_class_number(11.5)


def test_pitchtools_PitchClass_is_pitch_class_number_02():

    assert not pitchtools.PitchClass.is_pitch_class_number(-1)
    assert not pitchtools.PitchClass.is_pitch_class_number(-0.5)
    assert not pitchtools.PitchClass.is_pitch_class_number(12)
    assert not pitchtools.PitchClass.is_pitch_class_number(99)
    assert not pitchtools.PitchClass.is_pitch_class_number('foo')



# -*- coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import pitchtools
from abjad.tools.scoretools.Leaf import Leaf
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_


class Chord(Leaf):
    r'''A chord.

    ..  container:: example

        ::

            >>> chord = Chord("<e' cs'' f''>4")
            >>> show(chord) # doctest: +SKIP

        ..  doctest::

            >>> f(chord)
            <e' cs'' f''>4

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Leaves'

    __slots__ = (
        '_note_heads',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments):
        from abjad.ly import drums
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import parse
        assert len(arguments) in (0, 1, 2)
        self._note_heads = scoretools.NoteHeadList(
            client=self,
            )
        if len(arguments) == 1 and isinstance(arguments[0], str):
            string = '{{ {} }}'.format(arguments[0])
            parsed = parse(string)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            arguments = [parsed[0]]
        are_cautionary = []
        are_forced = []
        are_parenthesized = []
        if len(arguments) == 1 and isinstance(arguments[0], Leaf):
            leaf = arguments[0]
            written_pitches = []
            written_duration = leaf.written_duration
            if 'written_pitch' in dir(leaf):
                written_pitches.append(leaf.note_head.written_pitch)
                are_cautionary = [leaf.note_head.is_cautionary]
                are_forced = [leaf.note_head.is_forced]
                are_parenthesized = [leaf.note_head.is_parenthesized]
            elif 'written_pitches' in dir(leaf):
                written_pitches.extend(x.written_pitch
                    for x in leaf.note_heads)
                are_cautionary = [x.is_cautionary for x in leaf.note_heads]
                are_forced = [x.is_forced for x in leaf.note_heads]
                are_parenthesized = [x.is_parenthesized for x in
                    leaf.note_heads]
        elif len(arguments) == 2:
            written_pitches, written_duration = arguments
            if isinstance(written_pitches, str):
                written_pitches = [x for x in written_pitches.split() if x]
            elif isinstance(written_pitches, type(self)):
                written_pitches = written_pitches.written_pitches
        elif len(arguments) == 0:
            written_pitches = [0, 4, 7]
            written_duration = durationtools.Duration(1, 4)
        else:
            message = 'can not initialize chord from {!r}.'
            message = message.format(arguments)
            raise ValueError(message)
        Leaf.__init__(self, written_duration)
        if not are_cautionary:
            are_cautionary = [None] * len(written_pitches)
        if not are_forced:
            are_forced = [None] * len(written_pitches)
        if not are_parenthesized:
            are_parenthesized = [None] * len(written_pitches)
        for written_pitch, is_cautionary, is_forced, is_parenthesized in zip(
            written_pitches, are_cautionary, are_forced, are_parenthesized):
            if not is_cautionary:
                is_cautionary = None
            if not is_forced:
                is_forced = None
            if not is_parenthesized:
                is_parenthesized = None
            if written_pitch not in drums:
                note_head = scoretools.NoteHead(
                    written_pitch=written_pitch,
                    is_cautionary=is_cautionary,
                    is_forced=is_forced,
                    is_parenthesized=is_parenthesized,
                    )
            else:
                note_head = scoretools.DrumNoteHead(
                    written_pitch=written_pitch,
                    is_cautionary=is_cautionary,
                    is_forced=is_forced,
                    is_parenthesized=is_parenthesized,
                    )
            self._note_heads.append(note_head)
        if len(arguments) == 1 and isinstance(arguments[0], Leaf):
            self._copy_override_and_set_from_leaf(arguments[0])

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self.written_pitches, self.written_duration)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        return '<{}>{}'.format(self._summary, self._formatted_duration)

    @property
    def _compact_representation_with_tie(self):
        logical_tie = self._get_logical_tie()
        if 1 < len(logical_tie) and self is not logical_tie[-1]:
            return '{} ~'.format(self._body[0])
        else:
            return self._body[0]

    @property
    def _summary(self):
        return ' '.join([str(x) for x in self.note_heads])

    ### PRIVATE METHODS ###

    @staticmethod
    def _cast_defective_chord(chord):
        from abjad.tools import scoretools
        if isinstance(chord, Chord):
            note_head_count = len(chord.note_heads)
            if not note_head_count:
                return scoretools.Rest(chord)
            elif note_head_count == 1:
                return scoretools.Note(chord)
        return chord

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Leaf._copy_with_indicators_but_without_children_or_spanners(self)
        new.note_heads[:] = []
        for note_head in self.note_heads:
            new_note_head = copy.copy(note_head)
            new.note_heads.append(new_note_head)
        return new

    def _divide(self, pitch=None):
        from abjad.tools import scoretools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        pitch = pitch or pitchtools.NamedPitch('b', 3)
        pitch = pitchtools.NamedPitch(pitch)
        treble = copy.copy(self)
        bass = copy.copy(self)
        detach(markuptools.Markup, treble)
        detach(markuptools.Markup, bass)
        if isinstance(treble, scoretools.Note):
            if treble.written_pitch < pitch:
                treble = scoretools.Rest(treble)
        elif isinstance(treble, scoretools.Rest):
            pass
        elif isinstance(treble, scoretools.Chord):
            for note_head in reversed(treble.note_heads):
                if note_head.written_pitch < pitch:
                    treble.note_heads.remove(note_head)
        else:
            raise TypeError
        if isinstance(bass, scoretools.Note):
            if pitch <= bass.written_pitch:
                bass = scoretools.Rest(bass)
        elif isinstance(bass, scoretools.Rest):
            pass
        elif isinstance(bass, scoretools.Chord):
            for note_head in reversed(bass.note_heads):
                if pitch <= note_head.written_pitch:
                    bass.note_heads.remove(note_head)
        else:
            raise TypeError
        treble = self._cast_defective_chord(treble)
        bass = self._cast_defective_chord(bass)
        up_markup = self._get_markup(direction=Up)
        up_markup = [copy.copy(markup) for markup in up_markup]
        down_markup = self._get_markup(direction=Down)
        down_markup = [copy.copy(markup) for markup in down_markup]
        for markup in up_markup:
            markup(treble)
        for markup in down_markup:
            markup(bass)
        return treble, bass

    def _format_before_slot(self, bundle):
        result = []
        result.append(self._format_grace_body())
        result.append(('comments', bundle.before.comments))
        commands = bundle.before.commands
        if inspect_(self).has_indicator(indicatortools.Tremolo):
            tremolo_command = self._format_repeat_tremolo_command()
            commands = list(commands)
            commands.append(tremolo_command)
            commands = tuple(commands)
        result.append(('commands', commands))
        result.append(('indicators', bundle.before.indicators))
        result.append(('grob overrides', bundle.grob_overrides))
        result.append(('context settings', bundle.context_settings))
        result.append(('spanners', bundle.before.spanners))
        return result

    def _format_close_brackets_slot(self, bundle):
        result = []
        if inspect_(self).has_indicator(indicatortools.Tremolo):
            brackets_close = ['}']
            result.append([('close brackets', ''), brackets_close])
        return result

    def _format_leaf_nucleus(self):
        from abjad.tools import systemtools
        indent = systemtools.LilyPondFormatManager.indent
        result = []
        note_heads = self.note_heads
        if any('\n' in format(x) for x in note_heads):
            for note_head in note_heads:
                current_format = format(note_head)
                format_list = current_format.split('\n')
                format_list = [indent + x for x in format_list]
                result.extend(format_list)
            result.insert(0, '<')
            result.append('>')
            result = '\n'.join(result)
            result += str(self._formatted_duration)
        elif inspect_(self).has_indicator(indicatortools.Tremolo):
            reattack_duration = self._get_tremolo_reattack_duration()
            duration_string = reattack_duration.lilypond_duration_string
            durated_pitches = []
            for note_head in note_heads:
                durated_pitch = format(note_head) + duration_string
                durated_pitches.append(durated_pitch)
            tremolo = inspect_(self).get_indicator(indicatortools.Tremolo)
            if tremolo.is_slurred:
                durated_pitches[0] = durated_pitches[0] + r' \('
                durated_pitches[-1] = durated_pitches[-1] + r' \)'
            result = ' '.join(durated_pitches)
        else:
            result.extend([format(_) for _ in note_heads])
            result = '<%s>%s' % (' '.join(result), self._formatted_duration)
        # single string, but wrapped in list bc contribution
        return ['nucleus', [result]]

    def _format_open_brackets_slot(self, bundle):
        result = []
        if inspect_(self).has_indicator(indicatortools.Tremolo):
            brackets_open = ['{']
            result.append([('open brackets', ''), brackets_open])
        return result

    def _format_repeat_tremolo_command(self):
        tremolo = inspect_(self).get_indicator(indicatortools.Tremolo)
        reattack_duration = self._get_tremolo_reattack_duration()
        repeat_count = self.written_duration / reattack_duration / 2
        if not mathtools.is_integer_equivalent(repeat_count):
            message = 'can not tremolo duration {} with {} beams.'
            message = message.format(self.written_duration, tremolo.beam_count)
            raise Exception(message)
        repeat_count = int(repeat_count)
        command = r'\repeat tremolo {}'.format(repeat_count)
        return command

    def _get_lilypond_format(self):
        return super(Chord, self)._get_lilypond_format()

    def _get_sounding_pitches(self):
        from abjad.tools import instrumenttools
        from abjad.tools import pitchtools
        if 'sounding pitch' in inspect_(self).get_indicators(str):
            return self.written_pitches
        else:
            instrument = self._get_effective(
                instrumenttools.Instrument)
            if instrument:
                sounding_pitch = instrument.sounding_pitch_of_written_middle_c
            else:
                sounding_pitch = pitchtools.NamedPitch('C4')
            interval = pitchtools.NamedPitch('C4') - sounding_pitch
            sounding_pitches = [
                interval.transpose(pitch)
                for pitch in self.written_pitches
                ]
            return tuple(sounding_pitches)

    def _get_tremolo_reattack_duration(self):
        tremolos = inspect_(self).get_indicators(indicatortools.Tremolo)
        if not tremolos:
            return
        tremolo = tremolos[0]
        exponent = 2 + tremolo.beam_count
        denominator = 2 ** exponent
        reattack_duration = durationtools.Duration(1, denominator)
        return reattack_duration

    ### PUBLIC PROPERTIES ###

    @property
    def note_heads(self):
        r'''Gets note-heads in chord.

        ..  container:: example

            Gets note-heads in chord:

            ::

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> f(chord.note_heads)
                scoretools.NoteHeadList(
                    [
                        scoretools.NoteHead(
                            written_pitch=pitchtools.NamedPitch("g'"),
                            ),
                        scoretools.NoteHead(
                            written_pitch=pitchtools.NamedPitch("c''"),
                            ),
                        scoretools.NoteHead(
                            written_pitch=pitchtools.NamedPitch("e''"),
                            ),
                        ]
                    )

        ..  container:: example

            SetS note-heads with pitch names:

            ::

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.note_heads = "c' d' fs'"
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> f(chord)
                <c' d' fs'>4

        ..  container:: example

            Sets note-heads with pitch numbers:

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.note_heads = [16, 17, 19]
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> f(chord)
                <e'' f'' g''>4

        Set note-heads with any iterable.

        Returns note-head list.
        '''
        return self._note_heads

    @note_heads.setter
    def note_heads(self, note_heads):
        self._note_heads[:] = []
        if isinstance(note_heads, str):
            note_heads = note_heads.split()
        self.note_heads.extend(note_heads)

    @property
    def written_duration(self):
        r'''Gets written duration of chord.

        ..  container:: example

            **Example 1.** Get written duration:

            ::

                >>> chord = Chord("<e' cs'' f''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.written_duration
                Duration(1, 4)

        ..  container:: example

            **Example 2.** Set written duration:

            ::

                >>> chord = Chord("<e' cs'' f''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.written_duration = Duration(1, 16)
                >>> show(chord) # doctest: +SKIP

        Set duration.

        Returns duration.
        '''
        return Leaf.written_duration.fget(self)

    @written_duration.setter
    def written_duration(self, argument):
        Leaf.written_duration.fset(self, argument)

    @property
    def written_pitches(self):
        r'''Written pitches in chord.

        ..  container:: example

            **Example 1.** Get written pitches:

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.written_pitches
                PitchSegment("g' c'' e''")

        ..  container:: example

            **Example 2.** Set written pitches with pitch names:

            ::

                >>> chord = Chord("<e' g' c''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.written_pitches = "f' b' d''"
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> f(chord)
                <f' b' d''>4

            ::

                >>> chord.written_pitches
                PitchSegment("f' b' d''")

        Set written pitches with any iterable.

        Returns tuple.
        '''
        return pitchtools.PitchSegment(
            items=(note_head.written_pitch for note_head in self.note_heads),
            item_class=pitchtools.NamedPitch,
            )

    @written_pitches.setter
    def written_pitches(self, pitches):
        self.note_heads = pitches



from abjad import *


def test_scoretools_Component__get_nth_component_in_time_order_from_01():

    staff = Staff(r"c'4 \times 2/3 { d'8 e'8 f'8 } g'2")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4
            \times 2/3 {
                d'8
                e'8
                f'8
            }
            g'2
        }
        '''
        )

    leaves = select(staff).by_leaf()
    tuplet = staff[1]

    assert leaves[0]._get_nth_component_in_time_order_from(-1) is None
    assert leaves[0]._get_nth_component_in_time_order_from(0) is leaves[0]
    assert leaves[0]._get_nth_component_in_time_order_from(1) is tuplet
    assert leaves[0]._get_nth_component_in_time_order_from(2) is leaves[4]
    assert leaves[0]._get_nth_component_in_time_order_from(3) is None

    assert tuplet._get_nth_component_in_time_order_from(-2) is None
    assert tuplet._get_nth_component_in_time_order_from(-1) is leaves[0]
    assert tuplet._get_nth_component_in_time_order_from(0) is tuplet
    assert tuplet._get_nth_component_in_time_order_from(1) is leaves[4]

    assert leaves[1]._get_nth_component_in_time_order_from(-1) is leaves[0]
    assert leaves[1]._get_nth_component_in_time_order_from(0) is leaves[1]
    assert leaves[1]._get_nth_component_in_time_order_from(1) is leaves[2]
    assert leaves[1]._get_nth_component_in_time_order_from(2) is leaves[3]
    assert leaves[1]._get_nth_component_in_time_order_from(3) is leaves[4]
    assert leaves[1]._get_nth_component_in_time_order_from(4) is None



# -*- coding: utf-8 -*-
from __future__ import print_function
import inspect
import os
import types
from abjad.tools import stringtools
from abjad.tools.commandlinetools.CommandlineScript import CommandlineScript


class StatsScript(CommandlineScript):
    r'''Builds statistics about a codebase.

    ..  shell::

        ajv stats  --help

    '''

    ### CLASS VARIABLES ###

    alias = 'stats'
    short_description = 'Build statistics about Python modules in PATH.'

    ### PRIVATE METHODS ###

    def _iterate_module(self, module):
        results = []
        for name in dir(module):
            obj = getattr(module, name)
            if not isinstance(obj, (type, types.FunctionType)):
                continue
            elif obj.__module__ != module.__name__:
                continue
            results.append(obj)
        return results

    def _print_results(self, counts):
        template = stringtools.normalize('''
        Source lines: {source_lines}
        Public classes: {public_classes}
            Unique public methods: {unique_public_methods}
            Unique public properties: {unique_public_properties}
            Unique private methods: {unique_private_methods}
            Unique private properties: {unique_private_properties}
        Public functions: {public_functions}
        Private classes: {private_classes}
        Private functions: {private_functions}
        ''')
        result = template.format(
            source_lines=counts['source_lines'],
            public_classes=counts['public_classes'],
            unique_public_methods=counts['unique_public_methods'],
            unique_public_properties=counts['unique_public_properties'],
            unique_private_methods=counts['unique_private_methods'],
            unique_private_properties=counts['unique_private_properties'],
            public_functions=counts['public_functions'],
            private_classes=counts['private_classes'],
            private_functions=counts['private_functions'],
            )
        print(result)

    def _process_args(self, arguments):
        from abjad.tools import documentationtools
        path = arguments.path
        if not os.path.isdir(path):
            path = os.path.dirname(path)
        counts = self._setup_counts()
        for module in documentationtools.yield_all_modules(
            code_root=path,
            ignored_file_names=[],
            ):
            with open(module.__file__, 'r') as file_pointer:
                contents = file_pointer.read()
                counts['source_lines'] += contents.count('\n')
            for obj in self._iterate_module(module):
                if isinstance(obj, types.FunctionType):
                    if obj.__name__.startswith('_'):
                        counts['private_functions'] += 1
                    else:
                        counts['public_functions'] += 1
                elif isinstance(obj, type):
                    if obj.__name__.startswith('_'):
                        counts['private_classes'] += 1
                    else:
                        counts['public_classes'] += 1
                        for attr in inspect.classify_class_attrs(obj):
                            if attr.defining_class != obj:
                                continue
                            if attr.kind in ('method', 'class method', 'static method'):
                                if attr.name.startswith('_'):
                                    counts['unique_private_methods'] += 1
                                else:
                                    counts['unique_public_methods'] += 1
                            elif attr.kind in ('property,'):
                                if attr.name.startswith('_'):
                                    counts['unique_private_properties'] += 1
                                else:
                                    counts['unique_public_properties'] += 1

        self._print_results(counts)

    def _setup_argument_parser(self, parser):
        parser.add_argument(
            'path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            type=self._validate_path,
            )

    def _setup_counts(self):
        counts = {
            'source_lines': 0,
            'private_classes': 0,
            'private_functions': 0,
            'public_classes': 0,
            'public_functions': 0,
            'unique_public_methods': 0,
            'unique_public_properties': 0,
            'unique_private_methods': 0,
            'unique_private_properties': 0,
            }
        return counts



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Octave_from_pitch_name_01():

    assert pitchtools.Octave.from_pitch_name("cs'") == 4
    assert pitchtools.Octave.from_pitch_name('cs') == 3
    assert pitchtools.Octave.from_pitch_name('cs,') == 2



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.datastructuretools.TypedCounter import TypedCounter


class RotationCounter(TypedCounter):
    r'''A rotation counter.

    ::

        >>> counter = rhythmmakertools.RotationCounter(default=3)

    ::

        >>> counter['talea__counts']
        3

    ::

        >>> counter['talea__counts'] += 1
        >>> counter['talea__counts']
        4

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_autoincrement',
        '_default',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        autoincrement=None,
        default=None,
        items=None,
        **keywords
        ):
        TypedCounter.__init__(
            self,
            items=items,
            **keywords
            )
        if autoincrement is not None:
            autoincrement = bool(autoincrement)
        self._autoincrement = autoincrement
        if default is not None:
            default = int(default or 0)
        self._default = default

    ### SPECIAL METHODS ###

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        Returns item or slice.
        '''
        argument = self._item_coercer(argument)
        if argument not in self._collection:
            self._collection[argument] = self._default or 0
        return self._collection.__getitem__(argument)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        names.extend(sorted(self._collection.keys()))
        if 'items' in names:
            names.remove('items')
        if not self.autoincrement:
            names.remove('autoincrement')
        return systemtools.FormatSpecification(
            self,
            repr_is_indented=False,
            storage_format_args_values=[],
            storage_format_kwargs_names=names,
            template_names=names,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def autoincrement(self):
        r'''Is true if rotation counter should be auto-incremented.
        Otherwise false.

        Returns true or false.
        '''
        return self._autoincrement

    @property
    def default(self):
        r'''Gets default count.

        Returns integer.
        '''
        return self._default
